(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('tslib'), require('@ts-ioc/core'), require('reflect-metadata')) :
	typeof define === 'function' && define.amd ? define(['tslib', '@ts-ioc/core', 'reflect-metadata'], factory) :
	(global.bootstrap = global.bootstrap || {}, global.bootstrap.umd = global.bootstrap.umd || {}, global.bootstrap.umd.js = factory(global.tslib,global['@ts-ioc/core'],global.Reflect));
}(this, (function (tslib_1,core_1,reflectMetadata) { 'use strict';

tslib_1 = tslib_1 && tslib_1.hasOwnProperty('default') ? tslib_1['default'] : tslib_1;
core_1 = core_1 && core_1.hasOwnProperty('default') ? core_1['default'] : core_1;
reflectMetadata = reflectMetadata && reflectMetadata.hasOwnProperty('default') ? reflectMetadata['default'] : reflectMetadata;

function unwrapExports (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var IAnnotationBuilder = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


var annoBuilderDesc = 'DI_AnnotationBuilder';
/**
 * inject Annotation class builder.
 *
 * @export
 * @class InjectBootstrapBuilder
 * @extends {Registration<T>}
 * @template T
 */
var InjectAnnotationBuilder = /** @class */ (function (_super) {
    tslib_1.__extends(InjectAnnotationBuilder, _super);
    function InjectAnnotationBuilder(type) {
        return _super.call(this, type, annoBuilderDesc) || this;
    }
    InjectAnnotationBuilder.classAnnations = { "name": "InjectAnnotationBuilder", "params": { "constructor": ["type"] } };
    return InjectAnnotationBuilder;
}(core_1.RefRegistration));
exports.InjectAnnotationBuilder = InjectAnnotationBuilder;
/**
 * Annotation class builder token.
 */
exports.AnnotationBuilderToken = new InjectAnnotationBuilder('');
/**
 * Default Annotation class builder token.
 */
exports.DefaultAnnotationBuilderToken = new InjectAnnotationBuilder('default');


});

unwrapExports(IAnnotationBuilder);
var IAnnotationBuilder_1 = IAnnotationBuilder.InjectAnnotationBuilder;
var IAnnotationBuilder_2 = IAnnotationBuilder.AnnotationBuilderToken;
var IAnnotationBuilder_3 = IAnnotationBuilder.DefaultAnnotationBuilderToken;

var Annotation = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * create type builder decorator
 *
 * @export
 * @template T
 * @param {string} name
 * @param {string} [decorType]
 * @param {(Token<IAnnotationBuilder<any>> | IAnnotationBuilder<any>)} [defaultBuilder]
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IAnnotationDecorator<T>}
 */
function createAnnotationDecorator(name, defaultBuilder, adapter, metadataExtends) {
    return core_1.createClassDecorator(name, function (args) {
        if (adapter) {
            adapter(args);
        }
    }, function (metadata) {
        if (metadataExtends) {
            metadata = metadataExtends(metadata);
        }
        if (defaultBuilder && !metadata.defaultAnnoBuilder) {
            metadata.defaultAnnoBuilder = defaultBuilder;
        }
        return metadata;
    });
}
exports.createAnnotationDecorator = createAnnotationDecorator;
/**
 * Annotation decorator, use to define class build way via config.
 *
 * @Annotation
 */
exports.Annotation = createAnnotationDecorator('Annotation', IAnnotationBuilder.AnnotationBuilderToken);


});

unwrapExports(Annotation);
var Annotation_1 = Annotation.createAnnotationDecorator;
var Annotation_2 = Annotation.Annotation;

var IModuleBuilder = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


var moduleBuilderDesc = 'DI_ModuleBuilder';
/**
 * inject module builder token.
 *
 * @export
 * @class InjectModuleBuilder
 * @extends {Registration<T>}
 * @template T
 */
var InjectModuleBuilderToken = /** @class */ (function (_super) {
    tslib_1.__extends(InjectModuleBuilderToken, _super);
    function InjectModuleBuilderToken(type) {
        return _super.call(this, type, moduleBuilderDesc) || this;
    }
    InjectModuleBuilderToken.classAnnations = { "name": "InjectModuleBuilderToken", "params": { "constructor": ["type"] } };
    return InjectModuleBuilderToken;
}(core_1.RefRegistration));
exports.InjectModuleBuilderToken = InjectModuleBuilderToken;
/**
 * default module builder token.
 */
exports.ModuleBuilderToken = new InjectModuleBuilderToken(Object);


});

unwrapExports(IModuleBuilder);
var IModuleBuilder_1 = IModuleBuilder.InjectModuleBuilderToken;
var IModuleBuilder_2 = IModuleBuilder.ModuleBuilderToken;

var DIModule = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });



/**
 * create bootstrap decorator.
 *
 * @export
 * @template T
 * @param {string} name decorator name.
 * @param {Token<IModuleBuilder>} [defaultBuilder]
 * @param {Token<IAnnotationBuilder<any>>} [defaultAnnoBuilder]
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IDIModuleDecorator<T>}
 */
function createDIModuleDecorator(name, defaultBuilder, defaultAnnoBuilder, adapter, metadataExtends) {
    return core_1.createClassDecorator(name, function (args) {
        if (adapter) {
            adapter(args);
        }
    }, function (metadata) {
        if (metadataExtends) {
            metadata = metadataExtends(metadata);
        }
        if (!metadata.name && core_1.isClass(metadata.token)) {
            var isuglify = /^[a-z]$/.test(metadata.token.name);
            if (isuglify && metadata.token.classAnnations) {
                metadata.name = metadata.token.classAnnations.name;
            }
            else {
                metadata.name = metadata.token.name;
            }
        }
        metadata.decorType = name;
        if (defaultBuilder && !metadata.defaultBuilder) {
            metadata.defaultBuilder = defaultBuilder;
        }
        if (defaultAnnoBuilder && !metadata.defaultAnnoBuilder) {
            metadata.defaultAnnoBuilder = defaultAnnoBuilder;
        }
        return metadata;
    });
}
exports.createDIModuleDecorator = createDIModuleDecorator;
/**
 * DIModule Decorator, definde class as DI module.
 *
 * @DIModule
 */
exports.DIModule = createDIModuleDecorator('DIModule', IModuleBuilder.ModuleBuilderToken, IAnnotationBuilder.AnnotationBuilderToken);


});

unwrapExports(DIModule);
var DIModule_1 = DIModule.createDIModuleDecorator;
var DIModule_2 = DIModule.DIModule;

var IConfigureManager = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 * configure manager token.
 */
exports.ConfigureMgrToken = new core_1.InjectToken('config-mgr');
/**
 * application default configuration token.
 */
exports.DefaultConfigureToken = new core_1.InjectToken('DI_Default_Configuration');
/**
 * configure loader token.
 */
exports.ConfigureLoaderToken = new core_1.InjectToken('DI_Configure_Loader');


});

unwrapExports(IConfigureManager);
var IConfigureManager_1 = IConfigureManager.ConfigureMgrToken;
var IConfigureManager_2 = IConfigureManager.DefaultConfigureToken;
var IConfigureManager_3 = IConfigureManager.ConfigureLoaderToken;

var ConfigureManager_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });



/**
 * configure manager.
 *
 * @export
 * @class ConfigureManager
 */
// @RefTo(RunnableBuilderToken)
var ConfigureManager = /** @class */ (function () {
    /**
     * Creates an instance of ConfigureManager.
     * @param {string} [baseURL]
     * @memberof ConfigureManager
     */
    function ConfigureManager(baseURL) {
        this.baseURL = baseURL;
        this.configs = [];
    }
    /**
     * use configuration.
     *
     * @param {(string | AppConfigure)} [config]
     * @returns {this} this configure manager.
     * @memberof Bootstrap
     */
    ConfigureManager.prototype.useConfiguration = function (config) {
        if (core_1.isUndefined(config)) {
            config = '';
        }
        // clean cached config.
        this.config = null;
        var idx = this.configs.indexOf(config);
        if (idx >= 0) {
            this.configs.splice(idx, 1);
        }
        this.configs.push(config);
        return this;
    };
    /**
     * bind runnable builder.
     *
     * @param {IRunnableBuilder<any>} builder
     * @memberof ConfigureManager
     */
    ConfigureManager.prototype.bindBuilder = function (builder) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/];
            });
        });
    };
    /**
     * get config.
     *
     * @returns {Promise<T>}
     * @memberof ConfigureManager
     */
    ConfigureManager.prototype.getConfig = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (!!this.config) return [3 /*break*/, 2];
                        _a = this;
                        return [4 /*yield*/, this.initConfig()];
                    case 1:
                        _a.config = _b.sent();
                        _b.label = 2;
                    case 2: return [2 /*return*/, this.config];
                }
            });
        });
    };
    /**
     * init config.
     *
     * @protected
     * @returns
     * @memberof ConfigureManager
     */
    ConfigureManager.prototype.initConfig = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var config, exts;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getDefaultConfig()];
                    case 1:
                        config = _a.sent();
                        if (this.configs.length < 1) {
                            this.configs.push(''); // load default loader config.
                        }
                        return [4 /*yield*/, Promise.all(this.configs.map(function (cfg) {
                                if (core_1.isString(cfg)) {
                                    return _this.loadConfig(cfg);
                                }
                                else {
                                    return cfg;
                                }
                            }))];
                    case 2:
                        exts = _a.sent();
                        exts.forEach(function (exCfg) {
                            if (exCfg) {
                                core_1.lang.assign(config, exCfg);
                            }
                        });
                        return [2 /*return*/, config];
                }
            });
        });
    };
    /**
     * load config.
     *
     * @protected
     * @param {string} src
     * @returns {Promise<T>}
     * @memberof ConfigureManager
     */
    ConfigureManager.prototype.loadConfig = function (src) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var loader, builder, cfg;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.container.has(IConfigureManager.ConfigureLoaderToken)) return [3 /*break*/, 2];
                        loader = this.container.resolve(IConfigureManager.ConfigureLoaderToken, { baseURL: this.baseURL, container: this.container });
                        return [4 /*yield*/, loader.load(src)];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2:
                        if (!src) return [3 /*break*/, 4];
                        builder = this.container.getBuilder();
                        return [4 /*yield*/, builder.loader.load([src])];
                    case 3:
                        cfg = _a.sent();
                        return [2 /*return*/, cfg.length ? cfg[0] : null];
                    case 4: return [2 /*return*/, null];
                }
            });
        });
    };
    /**
     * get default config.
     *
     * @protected
     * @returns {Promise<T>}
     * @memberof ConfigureManager
     */
    ConfigureManager.prototype.getDefaultConfig = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                if (this.container.has(IConfigureManager.DefaultConfigureToken)) {
                    return [2 /*return*/, this.container.resolve(IConfigureManager.DefaultConfigureToken)];
                }
                else {
                    return [2 /*return*/, {}];
                }
                return [2 /*return*/];
            });
        });
    };
    ConfigureManager.classAnnations = { "name": "ConfigureManager", "params": { "constructor": ["baseURL"], "useConfiguration": ["config"], "bindBuilder": ["builder"], "getConfig": [], "initConfig": [], "loadConfig": ["src"], "getDefaultConfig": [] } };
    tslib_1.__decorate([
        core_1.Inject(core_1.ContainerToken),
        tslib_1.__metadata("design:type", Object)
    ], ConfigureManager.prototype, "container", void 0);
    ConfigureManager = tslib_1.__decorate([
        core_1.Injectable(IConfigureManager.ConfigureMgrToken),
        tslib_1.__metadata("design:paramtypes", [String])
    ], ConfigureManager);
    return ConfigureManager;
}());
exports.ConfigureManager = ConfigureManager;


});

unwrapExports(ConfigureManager_1);
var ConfigureManager_2 = ConfigureManager_1.ConfigureManager;

var IRunnableBuilder = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 *  process run root.
 */
exports.ProcessRunRootToken = new core_1.InjectToken('ioc_processRunRoot');
/**
 * runable builder token.
 */
exports.RunnableBuilderToken = new core_1.InjectToken('DI_RunnableBuilder');


});

unwrapExports(IRunnableBuilder);
var IRunnableBuilder_1 = IRunnableBuilder.ProcessRunRootToken;
var IRunnableBuilder_2 = IRunnableBuilder.RunnableBuilderToken;

var DIModuleValidate = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });



/**
 * DIModule Validate Token
 */
exports.DIModuleValidateToken = new core_1.InjectModuleValidateToken(DIModule.DIModule.toString());
/**
 * DIModuel Validate
 *
 * @export
 * @class DIModuelValidate
 * @extends {ModuelValidate}
 */
var DIModuelValidate = /** @class */ (function (_super) {
    tslib_1.__extends(DIModuelValidate, _super);
    function DIModuelValidate() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    DIModuelValidate.prototype.getDecorator = function () {
        return DIModule.DIModule.toString();
    };
    DIModuelValidate.classAnnations = { "name": "DIModuelValidate", "params": { "getDecorator": [] } };
    DIModuelValidate = tslib_1.__decorate([
        core_1.Singleton(exports.DIModuleValidateToken)
    ], DIModuelValidate);
    return DIModuelValidate;
}(core_1.ModuelValidate));
exports.DIModuelValidate = DIModuelValidate;


});

unwrapExports(DIModuleValidate);
var DIModuleValidate_1 = DIModuleValidate.DIModuleValidateToken;
var DIModuleValidate_2 = DIModuleValidate.DIModuelValidate;

var ContainerPool_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 * container pool
 *
 * @export
 * @class ContainerPool
 */
var ContainerPool = /** @class */ (function () {
    function ContainerPool(containerBuilder) {
        this.containerBuilder = containerBuilder;
        this.pools = new Map();
    }
    ContainerPool.prototype.createContainer = function () {
        return this.containerBuilder.create();
    };
    ContainerPool.prototype.getTokenKey = function (token) {
        if (token instanceof core_1.Registration) {
            return token.toString();
        }
        return token;
    };
    ContainerPool.prototype.isDefault = function (container) {
        return container === this._default;
    };
    ContainerPool.prototype.hasDefault = function () {
        return !!this._default;
    };
    ContainerPool.prototype.getDefault = function () {
        if (!this._default) {
            this._default = this.createContainer();
        }
        return this._default;
    };
    ContainerPool.prototype.set = function (token, container) {
        var key = this.getTokenKey(token);
        if (this.pools.has(key)) {
            console.log(token.toString() + " module has loaded");
        }
        this.pools.set(key, container);
    };
    ContainerPool.prototype.get = function (token) {
        var key = this.getTokenKey(token);
        if (!this.has(key)) {
            return null;
        }
        return this.pools.get(token);
    };
    ContainerPool.prototype.has = function (token) {
        return this.pools.has(this.getTokenKey(token));
    };
    ContainerPool.prototype.create = function (parent) {
        parent = parent || this.getDefault();
        var container = parent.getBuilder().create();
        this.setParent(container, parent);
        return container;
    };
    ContainerPool.prototype.setParent = function (container, parent) {
        if (this.isDefault(container)) {
            return;
        }
        if (parent && parent !== container) {
            container.parent = parent;
        }
        else {
            container.parent = this.getDefault();
        }
    };
    ContainerPool.classAnnations = { "name": "ContainerPool", "params": { "constructor": ["containerBuilder"], "createContainer": [], "getTokenKey": ["token"], "isDefault": ["container"], "hasDefault": [], "getDefault": [], "set": ["token", "container"], "get": ["token"], "has": ["token"], "create": ["parent"], "setParent": ["container", "parent"] } };
    return ContainerPool;
}());
exports.ContainerPool = ContainerPool;
/**
 *  container pool token.
 */
exports.ContainerPoolToken = new core_1.InjectToken('DI_ContainerPool');


});

unwrapExports(ContainerPool_1);
var ContainerPool_2 = ContainerPool_1.ContainerPool;
var ContainerPool_3 = ContainerPool_1.ContainerPoolToken;

var Events_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 * custom events.
 *
 * @export
 * @class Events
 */
var Events = /** @class */ (function () {
    function Events() {
        this.eventsMap = {};
    }
    Events.prototype.on = function (name, event) {
        this.eventsMap[name] = this.eventsMap[name] || [];
        if (this.eventsMap[name].indexOf(event) < 0) {
            this.eventsMap[name].push(event);
        }
        return this;
    };
    Events.prototype.once = function (name, event) {
        var _this = this;
        var onceEvent = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            event.apply(void 0, args);
            _this.off(name, onceEvent);
        };
        this.on(name, onceEvent);
    };
    Events.prototype.off = function (name, event) {
        if (this.eventsMap[name]) {
            if (event) {
                this.eventsMap[name].splice(this.eventsMap[name].indexOf(event), 1);
            }
            else {
                delete this.eventsMap[name];
            }
        }
        return this;
    };
    Events.prototype.emit = function (name) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var events = this.eventsMap[name];
        if (core_1.isArray(events)) {
            events.forEach(function (ev) {
                ev.apply(void 0, args);
            });
        }
    };
    Events.classAnnations = { "name": "Events", "params": { "constructor": [], "on": ["name", "event"], "once": ["name", "event"], "off": ["name", "event"], "emit": ["name", "args"] } };
    return Events;
}());
exports.Events = Events;


});

unwrapExports(Events_1);
var Events_2 = Events_1.Events;

var types = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * is target di module class or not.
 *
 * @export
 * @param {*} target
 * @returns
 */
function isDIModuleClass(target) {
    return core_1.isClass(target) && core_1.hasOwnClassMetadata(DIModule.DIModule, target);
}
exports.isDIModuleClass = isDIModuleClass;


});

unwrapExports(types);
var types_1 = types.isDIModuleClass;

var utils = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(ContainerPool_1, exports);
tslib_1.__exportStar(Events_1, exports);
tslib_1.__exportStar(types, exports);


});

unwrapExports(utils);

var InjectedModule_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * injected module.
 *
 * @export
 * @class InjectedModule
 * @template T
 */
var InjectedModule = /** @class */ (function () {
    function InjectedModule(token, config, container, type, exports) {
        this.token = token;
        this.config = config;
        this.container = container;
        this.type = type;
        this.exports = exports;
    }
    InjectedModule.prototype.getProviderMap = function () {
        if (!this._map) {
            this._map = this.container.resolveValue(new core_1.InjectReference(core_1.ProviderMap, this.type || this.token));
        }
        return this._map;
    };
    InjectedModule.prototype.resolve = function (token) {
        var providers = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            providers[_i - 1] = arguments[_i];
        }
        var _a;
        var pdr = this.getProviderMap();
        if (pdr && pdr.hasRegister(token)) {
            return pdr.resolve.apply(pdr, [token].concat(providers));
        }
        else {
            return (_a = this.container).resolveValue.apply(_a, [token].concat(providers));
        }
    };
    InjectedModule.prototype.hasRegister = function (key) {
        if (this.container.hasRegister(key)) {
            return true;
        }
        else {
            var pdr = this.getProviderMap();
            return pdr && pdr.hasRegister(key);
        }
    };
    InjectedModule.classAnnations = { "name": "InjectedModule", "params": { "constructor": ["token", "config", "container", "type", "exports"], "getProviderMap": [], "resolve": ["token", "providers"], "hasRegister": ["key"] } };
    return InjectedModule;
}());
exports.InjectedModule = InjectedModule;
/**
 * Injected Module Token.
 *
 * @export
 * @class InjectModuleMetaConfigToken
 * @extends {Registration<Type<T>>}
 * @template T
 */
var InjectedModuleToken = /** @class */ (function (_super) {
    tslib_1.__extends(InjectedModuleToken, _super);
    function InjectedModuleToken(type) {
        return _super.call(this, type, 'InjectedModule') || this;
    }
    InjectedModuleToken.classAnnations = { "name": "InjectedModuleToken", "params": { "constructor": ["type"] } };
    return InjectedModuleToken;
}(core_1.Registration));
exports.InjectedModuleToken = InjectedModuleToken;


});

unwrapExports(InjectedModule_1);
var InjectedModule_2 = InjectedModule_1.InjectedModule;
var InjectedModule_3 = InjectedModule_1.InjectedModuleToken;

var DIModuleInjector_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });






/**
 * DIModule injector token.
 */
exports.DIModuleInjectorToken = new core_1.InjectModuleInjectorToken(DIModule.DIModule.toString());
/**
 * DIModule injector.
 *
 * @export
 * @class DIModuleInjector
 * @extends {ModuleInjector}
 */
var DIModuleInjector = /** @class */ (function (_super) {
    tslib_1.__extends(DIModuleInjector, _super);
    function DIModuleInjector(validate) {
        return _super.call(this, validate) || this;
    }
    DIModuleInjector.prototype.setup = function (container, type) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.importModule(container, type)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    DIModuleInjector.prototype.syncSetup = function (container, type) {
        this.importModule(container, type);
    };
    DIModuleInjector.prototype.valid = function (container, type) {
        if (!this.validate) {
            return true;
        }
        return this.validate.valid(type);
    };
    DIModuleInjector.prototype.getMetaAccessor = function (container, token) {
        return container.getService(core_1.MetaAccessorToken, token);
    };
    DIModuleInjector.prototype.importModule = function (container, type) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var pools, newContainer, builder, decorator, accor, metaConfig, exps, classProvides, injMd;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        pools = container.get(utils.ContainerPoolToken);
                        newContainer = pools.create(container);
                        newContainer.register(type);
                        builder = newContainer.getBuilder();
                        decorator = this.validate.getDecorator();
                        accor = this.getMetaAccessor(newContainer, decorator);
                        metaConfig = accor.getMetadata(type, newContainer, undefined, decorator ? function (dec) { return dec === decorator; } : undefined);
                        return [4 /*yield*/, this.registerConfgureDepds(newContainer, metaConfig, type)];
                    case 1:
                        metaConfig = _a.sent();
                        exps = [].concat.apply([], builder.loader.getTypes(metaConfig.exports || []));
                        classProvides = [];
                        exps.forEach(function (ty) {
                            var classPd = newContainer.resolveValue(new core_1.InjectClassProvidesToken(ty));
                            if (classPd && core_1.isArray(classPd.provides) && classPd.provides.length) {
                                classProvides = classProvides.concat(classPd.provides);
                            }
                        });
                        injMd = new InjectedModule_1.InjectedModule(metaConfig.token || type, metaConfig, newContainer, type, exps);
                        container.bindProvider(new InjectedModule_1.InjectedModuleToken(type), injMd);
                        return [4 /*yield*/, this.importConfigExports(container, newContainer, injMd)];
                    case 2:
                        _a.sent();
                        return [2 /*return*/, injMd];
                }
            });
        });
    };
    DIModuleInjector.prototype.registerConfgureDepds = function (container, config, type) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(core_1.isArray(config.imports) && config.imports.length)) return [3 /*break*/, 2];
                        return [4 /*yield*/, container.loadModule.apply(container, config.imports)];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2:
                        if (!(!type && core_1.isArray(config.providers) && config.providers.length)) return [3 /*break*/, 4];
                        return [4 /*yield*/, this.bindProvider(container, config.providers)];
                    case 3:
                        _a.sent();
                        _a.label = 4;
                    case 4: return [2 /*return*/, config];
                }
            });
        });
    };
    DIModuleInjector.prototype.importConfigExports = function (container, providerContainer, injMd) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var chain_1;
            return tslib_1.__generator(this, function (_a) {
                if (container === providerContainer) {
                    return [2 /*return*/, container];
                }
                if (injMd) {
                    chain_1 = container.getResolvers();
                    chain_1.next(injMd);
                    if (injMd.exports && injMd.exports.length) {
                        providerContainer.getResolvers().toArray().filter(function (r) {
                            if (r instanceof core_1.Container) {
                                return false;
                            }
                            else {
                                return injMd.exports.indexOf(r.type) >= 0;
                            }
                        }).forEach(function (r) {
                            chain_1.next(r);
                        });
                    }
                }
                return [2 /*return*/, container];
            });
        });
    };
    DIModuleInjector.prototype.bindProvider = function (container, providers) {
        var parser = container.getProviderParser();
        var pdrmap = parser.parse.apply(parser, providers);
        var tokens = pdrmap.provides();
        tokens.forEach(function (key) {
            container.bindProvider(key, function () {
                var providers = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    providers[_i] = arguments[_i];
                }
                return pdrmap.resolve.apply(pdrmap, [key].concat(providers));
            });
        });
        return tokens;
    };
    DIModuleInjector.classAnnations = { "name": "DIModuleInjector", "params": { "constructor": ["validate"], "setup": ["container", "type"], "syncSetup": ["container", "type"], "valid": ["container", "type"], "getMetaAccessor": ["container", "token"], "importModule": ["container", "type"], "registerConfgureDepds": ["container", "config", "type"], "importConfigExports": ["container", "providerContainer", "injMd"], "bindProvider": ["container", "providers"] } };
    DIModuleInjector = tslib_1.__decorate([
        core_1.Singleton(exports.DIModuleInjectorToken),
        tslib_1.__param(0, core_1.Inject(DIModuleValidate.DIModuleValidateToken)),
        tslib_1.__metadata("design:paramtypes", [Object])
    ], DIModuleInjector);
    return DIModuleInjector;
}(core_1.ModuleInjector));
exports.DIModuleInjector = DIModuleInjector;


});

unwrapExports(DIModuleInjector_1);
var DIModuleInjector_2 = DIModuleInjector_1.DIModuleInjectorToken;
var DIModuleInjector_3 = DIModuleInjector_1.DIModuleInjector;

var AnnoBuildStrategy_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


var InjectAnnoBuildStrategyToken = /** @class */ (function (_super) {
    tslib_1.__extends(InjectAnnoBuildStrategyToken, _super);
    function InjectAnnoBuildStrategyToken(type) {
        return _super.call(this, type, 'AnnoBuildStrategy') || this;
    }
    InjectAnnoBuildStrategyToken.classAnnations = { "name": "InjectAnnoBuildStrategyToken", "params": { "constructor": ["type"] } };
    return InjectAnnoBuildStrategyToken;
}(core_1.RefRegistration));
exports.InjectAnnoBuildStrategyToken = InjectAnnoBuildStrategyToken;
/**
 *  base AnnoBuildStrategy Token
 */
exports.AnnoBuildStrategyToken = new InjectAnnoBuildStrategyToken(Object);
/**
 * annotation instance build strategy.
 *
 * @export
 * @class AnnoBuildStrategy
 * @implements {IAnnoBuildStrategy<T>}
 * @template T
 */
var AnnoBuildStrategy = /** @class */ (function () {
    function AnnoBuildStrategy() {
    }
    AnnoBuildStrategy.prototype.build = function (instance, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!instance) {
                            return [2 /*return*/];
                        }
                        return [4 /*yield*/, this.beforeInit(instance, config)];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.process(instance, config, options)];
                    case 2:
                        _a.sent();
                        return [4 /*yield*/, this.afterInit(instance, config)];
                    case 3:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * after annotation instance init.
     *
     * @protected
     * @param {BootHooks<T>} instance
     * @param {AnnotationConfigure<T>} config
     * @returns {Promise<void>}
     * @memberof AnnoBuildStrategy
     */
    AnnoBuildStrategy.prototype.beforeInit = function (instance, config) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!core_1.isFunction(instance.anBeforeInit)) return [3 /*break*/, 2];
                        return [4 /*yield*/, instance.anBeforeInit(config)];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * build annotation instance process.
     *
     * @protected
     * @param {T} instance
     * @param {AnnotationConfigure<T>} config
     * @param {BuildOptions<T>} [options]
     * @returns {Promise<void>}
     * @memberof AnnoBuildStrategy
     */
    AnnoBuildStrategy.prototype.process = function (instance, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/];
            });
        });
    };
    /**
     * after annotation instance init.
     *
     * @protected
     * @param {BootHooks<T>} instance
     * @param {AnnotationConfigure<T>} config
     * @returns {Promise<void>}
     * @memberof AnnoBuildStrategy
     */
    AnnoBuildStrategy.prototype.afterInit = function (instance, config) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!core_1.isFunction(instance.anAfterInit)) return [3 /*break*/, 2];
                        return [4 /*yield*/, instance.anAfterInit(config)];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    AnnoBuildStrategy.classAnnations = { "name": "AnnoBuildStrategy", "params": { "build": ["instance", "config", "options"], "beforeInit": ["instance", "config"], "process": ["instance", "config", "options"], "afterInit": ["instance", "config"] } };
    AnnoBuildStrategy = tslib_1.__decorate([
        core_1.Singleton(exports.AnnoBuildStrategyToken)
    ], AnnoBuildStrategy);
    return AnnoBuildStrategy;
}());
exports.AnnoBuildStrategy = AnnoBuildStrategy;


});

unwrapExports(AnnoBuildStrategy_1);
var AnnoBuildStrategy_2 = AnnoBuildStrategy_1.InjectAnnoBuildStrategyToken;
var AnnoBuildStrategy_3 = AnnoBuildStrategy_1.AnnoBuildStrategyToken;
var AnnoBuildStrategy_4 = AnnoBuildStrategy_1.AnnoBuildStrategy;

var Runner_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * boot element.
 *
 * @export
 * @abstract
 * @class Boot
 * @implements {IBoot}
 */
var Runner = /** @class */ (function () {
    function Runner(token, instance, config) {
        this.token = token;
        this.instance = instance;
        this.config = config;
    }
    Runner.prototype.getTarget = function () {
        return this.instance;
    };
    Runner.prototype.getTargetToken = function () {
        return this.token || core_1.lang.getClass(this.instance);
    };
    Runner.classAnnations = { "name": "Runner", "params": { "constructor": ["token", "instance", "config"], "getTarget": [], "getTargetToken": [], "run": ["data"] } };
    return Runner;
}());
exports.Runner = Runner;
/**
 * boot element
 *
 * @export
 * @class Boot
 * @extends {Runner<any>}
 */
var Boot = /** @class */ (function (_super) {
    tslib_1.__extends(Boot, _super);
    function Boot(token, instance, config) {
        var _this = _super.call(this, token, instance, config) || this;
        _this.token = token;
        _this.instance = instance;
        _this.config = config;
        return _this;
    }
    Boot.classAnnations = { "name": "Boot", "params": { "constructor": ["token", "instance", "config"] } };
    return Boot;
}(Runner));
exports.Boot = Boot;
/**
 * target is runner or not.
 *
 * @export
 * @param {*} target
 * @returns {target is IRunner<any>}
 */
function isRunner(target) {
    if (target instanceof Runner) {
        return true;
    }
    if (target && core_1.isFunction(target.run)) {
        return true;
    }
    return false;
}
exports.isRunner = isRunner;


});

unwrapExports(Runner_1);
var Runner_2 = Runner_1.Runner;
var Runner_3 = Runner_1.Boot;
var Runner_4 = Runner_1.isRunner;

var Service_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 * base service.
 *
 * @export
 * @abstract
 * @class Service
 * @implements {IService}
 */
var Service = /** @class */ (function () {
    function Service(token, instance, config) {
        this.token = token;
        this.instance = instance;
        this.config = config;
    }
    /**
     * get target.
     *
     * @returns {T}
     * @memberof Service
     */
    Service.prototype.getTarget = function () {
        return this.instance;
    };
    Service.prototype.getTargetToken = function () {
        return this.token || core_1.lang.getClass(this.instance);
    };
    Service.classAnnations = { "name": "Service", "params": { "constructor": ["token", "instance", "config"], "getTarget": [], "getTargetToken": [], "start": ["data"], "stop": [] } };
    return Service;
}());
exports.Service = Service;
/**
 * target is service or not.
 *
 * @export
 * @param {*} target
 * @returns {target is IService<any>}
 */
function isService(target) {
    if (target instanceof Service) {
        return true;
    }
    if (target && core_1.isFunction(target.start)) {
        return true;
    }
    return false;
}
exports.isService = isService;


});

unwrapExports(Service_1);
var Service_2 = Service_1.Service;
var Service_3 = Service_1.isService;

var Runnable = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * application runner token.
 *
 * @export
 * @class InjectRunnerToken
 * @extends {Registration<IRunner<T>>}
 * @template T
 */
var InjectRunnableToken = /** @class */ (function (_super) {
    tslib_1.__extends(InjectRunnableToken, _super);
    function InjectRunnableToken(type) {
        return _super.call(this, type, 'Runnable') || this;
    }
    InjectRunnableToken.classAnnations = { "name": "InjectRunnableToken", "params": { "constructor": ["type"] } };
    return InjectRunnableToken;
}(core_1.RefRegistration));
exports.InjectRunnableToken = InjectRunnableToken;
/**
 * default service token.
 */
exports.ServiceToken = new InjectRunnableToken('service');
/**
 * default runner token.
 */
exports.RunnerToken = new InjectRunnableToken('runner');


});

unwrapExports(Runnable);
var Runnable_1 = Runnable.InjectRunnableToken;
var Runnable_2 = Runnable.ServiceToken;
var Runnable_3 = Runnable.RunnerToken;

var runnable = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(Runner_1, exports);
tslib_1.__exportStar(Service_1, exports);
tslib_1.__exportStar(Runnable, exports);


});

unwrapExports(runnable);

var AnnotationBuilder_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });





/**
 * Annotation class builder. build class with metadata and config.
 *
 * @export
 * @class AnnotationBuilder
 * @implements {implements IAnnotationBuilder<T>}
 * @template T
 */
var AnnotationBuilder = /** @class */ (function () {
    function AnnotationBuilder() {
    }
    AnnotationBuilder_1 = AnnotationBuilder;
    /**
     * get metadata accessor.
     *
     * @param {(Token<any>|| AnnotationConfigure<T>)} token
     * @param {AnnotationConfigure<T>} [config]
     * @returns {IMetaAccessor<any>}
     * @memberof AnnotationBuilder
     */
    AnnotationBuilder.prototype.getMetaAccessor = function (token, config) {
        var mtk;
        if (core_1.isToken(token)) {
            mtk = token;
        }
        else {
            config = token;
        }
        return this.container.getService(core_1.MetaAccessorToken, mtk ? [mtk, core_1.lang.getClass(this)] : core_1.lang.getClass(this), function (tk) { return new core_1.InjectMetaAccessorToken(tk); }, config ? (config.defaultMetaAccessor || core_1.MetaAccessorToken) : core_1.MetaAccessorToken);
    };
    /**
     * build token type with config.
     *
     * @param {(Token<T> | AnnotationConfigure<T>)} token
     * @param {(AnnotationConfigure<T> | BuildOptions<T>)} [config]
     * @param {BuildOptions<T>} [options]
     * @returns {Promise<T>}
     * @memberof AnnotationBuilder
     */
    AnnotationBuilder.prototype.build = function (token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var tk, cfg, accessor, builder, instance, strategy;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (core_1.isToken(token)) {
                            tk = token;
                            if (core_1.isUndefined(options)) {
                                options = config;
                                cfg = null;
                            }
                            else {
                                cfg = config;
                            }
                            cfg = this.getMetaAccessor(tk, cfg).getMetadata(tk, this.container, cfg);
                        }
                        else {
                            options = config;
                            cfg = token;
                            accessor = this.getMetaAccessor(cfg);
                            tk = accessor.getToken(cfg, this.container);
                            if (core_1.isToken(tk)) {
                                cfg = accessor.getMetadata(tk, this.container, cfg);
                            }
                        }
                        if (core_1.isClass(tk) && !this.container.hasRegister(tk)) {
                            this.container.register(tk);
                        }
                        if (options && core_1.isFunction(options.vaild) && !options.vaild(tk)) {
                            return [2 /*return*/, null];
                        }
                        builder = (options && options.builder) ? options.builder : this.getBuilder(tk, cfg);
                        if (!!this.isEqual(builder)) return [3 /*break*/, 2];
                        return [4 /*yield*/, builder.build(tk, cfg, core_1.lang.assign(options || {}, { builder: builder }))];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2: return [4 /*yield*/, this.registerExts(cfg)];
                    case 3:
                        _a.sent();
                        return [4 /*yield*/, this.createInstance(tk, cfg, options)];
                    case 4:
                        instance = _a.sent();
                        if (!instance) {
                            return [2 /*return*/, null];
                        }
                        strategy = this.container.getService(AnnoBuildStrategy_1.AnnoBuildStrategyToken, [
                            tk || core_1.lang.getClass(instance)
                        ].concat(((options && options.target) ? [{ target: core_1.lang.getClass(options.target), level: core_1.RefTagLevel.self }] : []), [
                            { target: core_1.lang.getClass(this), level: core_1.RefTagLevel.self }
                        ]), function (tk) { return new AnnoBuildStrategy_1.InjectAnnoBuildStrategyToken(tk); });
                        if (!strategy) return [3 /*break*/, 6];
                        return [4 /*yield*/, strategy.build(instance, cfg, options)];
                    case 5:
                        _a.sent();
                        _a.label = 6;
                    case 6:
                        if (options && core_1.isFunction(options.onCompleted)) {
                            options.onCompleted(tk, cfg, instance, this);
                        }
                        return [2 /*return*/, instance];
                }
            });
        });
    };
    /**
     * run runable.
     *
     * @param {(Token<T> | AnnotationConfigure<T>)} runable
     * @param {*} [data] bootstrap data, build data, Runnable data.
     * @returns {Promise<Runnable<T>>}
     * @memberof IGModuleBuilder
     */
    AnnotationBuilder.prototype.boot = function (runable, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var instance, builder, tk, cfg, onCompleted, runner, data;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        onCompleted = function (token, config, hook, builder) {
                            tk = token;
                            cfg = config;
                            instance = hook;
                            builder = builder;
                        };
                        if (!core_1.isToken(runable)) return [3 /*break*/, 2];
                        if (core_1.isUndefined(options)) {
                            options = config;
                            cfg = null;
                        }
                        else {
                            cfg = config;
                        }
                        return [4 /*yield*/, this.build(runable, cfg, core_1.lang.assign({
                                onCompleted: onCompleted
                            }, options))];
                    case 1:
                        _a.sent();
                        return [3 /*break*/, 4];
                    case 2: return [4 /*yield*/, this.build(runable, core_1.lang.assign({
                            onCompleted: onCompleted
                        }, config))];
                    case 3:
                        _a.sent();
                        _a.label = 4;
                    case 4:
                        builder = builder || this;
                        runner = builder.resolveRunable(instance, cfg, tk, options);
                        data = options ? options.data : undefined;
                        if (!runnable.isRunner(runner)) return [3 /*break*/, 6];
                        return [4 /*yield*/, runner.run(data)];
                    case 5:
                        _a.sent();
                        return [3 /*break*/, 8];
                    case 6:
                        if (!runnable.isService(runner)) return [3 /*break*/, 8];
                        return [4 /*yield*/, runner.start(data)];
                    case 7:
                        _a.sent();
                        _a.label = 8;
                    case 8: return [2 /*return*/, runner];
                }
            });
        });
    };
    /**
     * run annotation instance.
     *
     * @param {T} instance
     * @param {AnnotationConfigure<T>} [config]
     * @param {Token<T>} [token]
     * @returns {Promise<Runnable<T>>}
     * @memberof AnnotationBuilder
     */
    AnnotationBuilder.prototype.resolveRunable = function (instance, config, token, options) {
        var _a;
        if (!instance) {
            return null;
        }
        var tk;
        if (core_1.isToken(token)) {
            tk = token;
        }
        else {
            options = token;
        }
        tk = tk || core_1.lang.getClass(instance);
        if (!config) {
            config = this.getMetaAccessor(tk).getMetadata(tk, this.container);
        }
        if (instance instanceof runnable.Runner || instance instanceof runnable.Service) {
            return instance;
        }
        else {
            var providers = [{ provide: tk, useValue: instance }, { token: tk, instance: instance, config: config }];
            return (_a = this.container).getService.apply(_a, [[runnable.RunnerToken, runnable.ServiceToken], [
                    tk
                ].concat(((options && options.target) ? [{ target: core_1.lang.getClass(options.target), level: core_1.RefTagLevel.self }] : []), [
                    { target: core_1.lang.getClass(this), level: core_1.RefTagLevel.self }
                ]), function (tk) { return new runnable.InjectRunnableToken(tk); },
                config.defaultRunnable || true].concat(providers));
        }
    };
    AnnotationBuilder.prototype.createInstance = function (token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var instance;
            return tslib_1.__generator(this, function (_a) {
                if (!core_1.isToken(token)) {
                    return [2 /*return*/, null];
                }
                instance = this.resolveToken(token, options ? options.target : undefined);
                if (core_1.isNullOrUndefined(instance)) {
                    console.log("can not find token " + (token ? token.toString() : null) + " in container.");
                    return [2 /*return*/, null];
                }
                return [2 /*return*/, instance];
            });
        });
    };
    AnnotationBuilder.prototype.getBuilder = function (token, config, options) {
        var builder;
        if (config && config.annoBuilder) {
            if (core_1.isClass(config.annoBuilder)) {
                if (!this.container.has(config.annoBuilder)) {
                    this.container.register(config.annoBuilder);
                }
            }
            if (core_1.isToken(config.annoBuilder)) {
                builder = this.container.resolve(config.annoBuilder);
            }
            else if (config.annoBuilder instanceof AnnotationBuilder_1) {
                builder = config.annoBuilder;
            }
        }
        if (options && options.builder) {
            builder = options.builder;
        }
        if (!builder && token) {
            builder = this.container.getService(IAnnotationBuilder.AnnotationBuilderToken, [
                token
            ].concat((options && options.target) ? [{ target: core_1.lang.getClass(options.target), level: core_1.RefTagLevel.self }] : []), function (tk) { return new IAnnotationBuilder.InjectAnnotationBuilder(tk); }, config.defaultAnnoBuilder || IAnnotationBuilder.AnnotationBuilderToken);
        }
        if (builder && !builder.container) {
            builder.container = this.container;
        }
        return builder || this;
    };
    /**
     * register extension before create instance.
     *
     * @protected
     * @param {AnnotationConfigure<T>} [config]
     * @memberof AnnotationBuilder
     */
    AnnotationBuilder.prototype.registerExts = function (config) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/];
            });
        });
    };
    AnnotationBuilder.prototype.isEqual = function (build) {
        if (!build) {
            return false;
        }
        if (build === this) {
            return true;
        }
        if (core_1.lang.getClass(build) === core_1.lang.getClass(this)) {
            return true;
        }
        return false;
    };
    AnnotationBuilder.prototype.resolveToken = function (token, target) {
        if (target) {
            return this.container.getService(token, target);
        }
        return this.container.resolve(token);
    };
    var AnnotationBuilder_1;
    AnnotationBuilder.classAnnations = { "name": "AnnotationBuilder", "params": { "constructor": [], "getMetaAccessor": ["token", "config"], "build": ["token", "config", "options"], "boot": ["runable", "config", "options"], "resolveRunable": ["instance", "config", "token", "options"], "createInstance": ["token", "config", "options"], "getBuilder": ["token", "config", "options"], "registerExts": ["config"], "isEqual": ["build"], "resolveToken": ["token", "target"] } };
    tslib_1.__decorate([
        core_1.Inject(core_1.ContainerToken),
        tslib_1.__metadata("design:type", Object)
    ], AnnotationBuilder.prototype, "container", void 0);
    AnnotationBuilder = AnnotationBuilder_1 = tslib_1.__decorate([
        core_1.Singleton(IAnnotationBuilder.AnnotationBuilderToken),
        tslib_1.__metadata("design:paramtypes", [])
    ], AnnotationBuilder);
    return AnnotationBuilder;
}());
exports.AnnotationBuilder = AnnotationBuilder;


});

unwrapExports(AnnotationBuilder_1);
var AnnotationBuilder_2 = AnnotationBuilder_1.AnnotationBuilder;

var annotations = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(IAnnotationBuilder, exports);
tslib_1.__exportStar(AnnoBuildStrategy_1, exports);
tslib_1.__exportStar(AnnotationBuilder_1, exports);


});

unwrapExports(annotations);

var ModuleBuilder_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });







/**
 * inject module load token.
 *
 * @export
 * @class InjectModuleLoadToken
 * @extends {Registration<T>}
 * @template T
 */
var InjectModuleLoadToken = /** @class */ (function (_super) {
    tslib_1.__extends(InjectModuleLoadToken, _super);
    function InjectModuleLoadToken(token) {
        return _super.call(this, token, 'module_loader') || this;
    }
    InjectModuleLoadToken.classAnnations = { "name": "InjectModuleLoadToken", "params": { "constructor": ["token"] } };
    return InjectModuleLoadToken;
}(core_1.Registration));
exports.InjectModuleLoadToken = InjectModuleLoadToken;
/**
 * module builder
 *
 * @export
 * @class ModuleBuilder
 * @implements {IModuleBuilder}
 * @template T
 */
var ModuleBuilder = /** @class */ (function () {
    function ModuleBuilder() {
    }
    ModuleBuilder.prototype.getPools = function () {
        return this.pools;
    };
    ModuleBuilder.prototype.getInjectedModule = function (type) {
        return this.getPools().getDefault().get(new InjectedModule_1.InjectedModuleToken(type));
    };
    /**
     * get metadata accessor.
     *
     * @param {IContainer} container
     * @param {Token<any>} token
     * @param {ModuleConfigure} [config]
     * @returns {IMetaAccessor<any>}
     * @memberof ModuleBuilder
     */
    ModuleBuilder.prototype.getMetaAccessor = function (container, token, config) {
        var mtk;
        if (core_1.isToken(token)) {
            mtk = token;
        }
        else {
            config = token;
        }
        return container.getService(core_1.MetaAccessorToken, mtk ? [mtk, core_1.lang.getClass(this)] : core_1.lang.getClass(this), function (tk) { return new core_1.InjectMetaAccessorToken(tk); }, config ? (config.defaultMetaAccessor || core_1.MetaAccessorToken) : core_1.MetaAccessorToken);
    };
    /**
     * load module.
     *
     * @param {(Token<T> | ModuleConfigure)} token
     * @param {(ModuleConfig<T> | BootOptions<T>)} [config]
     * @param {BootOptions<T>} [options]
     * @returns {Promise<InjectedModule<T>>}
     * @memberof ModuleBuilder
     */
    ModuleBuilder.prototype.load = function (token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var params, env, parent, injmd;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        params = this.vaildParams(token, config, options);
                        env = params.options ? params.options.env : null;
                        if (env instanceof InjectedModule_1.InjectedModule) {
                            return [2 /*return*/, env];
                        }
                        return [4 /*yield*/, this.getParentContainer(env)];
                    case 1:
                        parent = _a.sent();
                        if (!params.token) return [3 /*break*/, 5];
                        return [4 /*yield*/, this.loadViaToken(params.token, parent)];
                    case 2:
                        injmd = _a.sent();
                        if (!params.config) return [3 /*break*/, 4];
                        return [4 /*yield*/, this.loadViaConfig(params.config, injmd.container || parent)];
                    case 3:
                        _a.sent();
                        _a.label = 4;
                    case 4: return [2 /*return*/, injmd];
                    case 5: return [4 /*yield*/, this.loadViaConfig(params.config, parent)];
                    case 6: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * bootstrap with module and config.
     *
     * @param {(Token<T> | ModuleConfig<T>)} token
     * @param {(ModuleConfig<T> | BootOptions<T>)} [config]
     * @param {BootOptions<T>} [options]
     * @returns {Promise<Runnable<T>>}
     * @memberof ModuleBuilder
     */
    ModuleBuilder.prototype.bootstrap = function (token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var params, injmdl, cfg, tk, container, accessor, bootToken, anBuilder, mdBuilder;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        params = this.vaildParams(token, config, options);
                        options = params.options || {};
                        if (!params.token) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.load(params.token, params.config, options)];
                    case 1:
                        injmdl = _a.sent();
                        return [3 /*break*/, 4];
                    case 2: return [4 /*yield*/, this.load(params.config, options)];
                    case 3:
                        injmdl = _a.sent();
                        _a.label = 4;
                    case 4:
                        options.env = injmdl;
                        cfg = injmdl.config;
                        tk = injmdl.token || injmdl.type;
                        container = injmdl.container;
                        accessor = this.getMetaAccessor(container, tk);
                        bootToken = accessor.getBootToken(cfg, container);
                        if (!bootToken) return [3 /*break*/, 6];
                        anBuilder = this.getAnnoBuilder(container, bootToken, cfg);
                        return [4 /*yield*/, anBuilder.boot(bootToken, cfg, options)];
                    case 5: return [2 /*return*/, _a.sent()];
                    case 6:
                        mdBuilder = this.getAnnoBuilder(container, tk, cfg);
                        return [4 /*yield*/, mdBuilder.boot(tk, cfg, options)];
                    case 7: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    ModuleBuilder.prototype.vaildParams = function (token, config, options) {
        var params = {};
        if (core_1.isToken(token)) {
            params.token = token;
            if (core_1.isUndefined(options)) {
                params.options = config;
            }
            else {
                params.config = config;
            }
        }
        else {
            params.config = token;
            params.options = config;
        }
        return params;
    };
    ModuleBuilder.prototype.loadViaToken = function (token, parent) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var injmdl, cfg;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.import(token, parent)];
                    case 1:
                        injmdl = _a.sent();
                        if (!injmdl) {
                            cfg = this.getMetaAccessor(parent, token).getMetadata(token, parent);
                            injmdl = new InjectedModule_1.InjectedModule(token, cfg, parent);
                        }
                        return [2 /*return*/, injmdl];
                }
            });
        });
    };
    ModuleBuilder.prototype.loadViaConfig = function (config, parent) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var injmd, token, cfg, parser, pdrmap_1;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        injmd = null;
                        token = this.getMetaAccessor(parent, config).getToken(config, parent);
                        if (!token) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.import(token, parent)];
                    case 1:
                        injmd = _a.sent();
                        if (!injmd) {
                            cfg = this.getMetaAccessor(parent, token).getMetadata(token, parent, config);
                            injmd = new InjectedModule_1.InjectedModule(token, cfg, parent);
                        }
                        return [3 /*break*/, 3];
                    case 2:
                        token = null;
                        _a.label = 3;
                    case 3:
                        if (!(core_1.isArray(config.imports) && config.imports.length)) return [3 /*break*/, 5];
                        return [4 /*yield*/, parent.loadModule.apply(parent, config.imports)];
                    case 4:
                        _a.sent();
                        _a.label = 5;
                    case 5:
                        if (core_1.isArray(config.providers) && config.providers.length) {
                            parser = parent.getProviderParser();
                            pdrmap_1 = parser.parse.apply(parser, config.providers);
                            pdrmap_1.keys().forEach(function (key) {
                                core_1.isToken(key) && parent.bindProvider(key, function () {
                                    var providers = [];
                                    for (var _i = 0; _i < arguments.length; _i++) {
                                        providers[_i] = arguments[_i];
                                    }
                                    return pdrmap_1.resolve.apply(pdrmap_1, [key].concat(providers));
                                });
                            });
                        }
                        if (injmd) {
                            injmd.config = core_1.lang.assign({}, injmd.config, config);
                        }
                        else {
                            injmd = new InjectedModule_1.InjectedModule(token, config, parent);
                        }
                        return [2 /*return*/, injmd];
                }
            });
        });
    };
    ModuleBuilder.prototype.import = function (token, parent) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var type, key;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        type = core_1.isClass(token) ? token : parent.getTokenImpl(token);
                        if (!core_1.isClass(type)) return [3 /*break*/, 3];
                        key = new InjectedModule_1.InjectedModuleToken(type);
                        if (!parent.has(key)) return [3 /*break*/, 1];
                        return [2 /*return*/, parent.get(key)];
                    case 1: return [4 /*yield*/, parent.loadModule(type)];
                    case 2:
                        _a.sent();
                        if (parent.has(key)) {
                            return [2 /*return*/, parent.get(key)];
                        }
                        _a.label = 3;
                    case 3: return [2 /*return*/, null];
                }
            });
        });
    };
    ModuleBuilder.prototype.getParentContainer = function (env) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var parent;
            return tslib_1.__generator(this, function (_a) {
                if (env) {
                    if (env instanceof core_1.Container) {
                        parent = env;
                    }
                    else if (env instanceof InjectedModule_1.InjectedModule) {
                        parent = env.container.parent;
                    }
                }
                if (!parent) {
                    parent = this.getPools().getDefault();
                }
                return [2 /*return*/, parent];
            });
        });
    };
    ModuleBuilder.prototype.getAnnoBuilder = function (container, token, config) {
        var builder;
        if (core_1.isClass(config.annoBuilder)) {
            if (!container.has(config.annoBuilder)) {
                container.register(config.annoBuilder);
            }
        }
        if (core_1.isToken(config.annoBuilder)) {
            builder = container.resolve(config.annoBuilder);
        }
        else if (config.annoBuilder instanceof annotations.AnnotationBuilder) {
            builder = config.annoBuilder;
        }
        if (!builder && token) {
            builder = container.getService(annotations.AnnotationBuilderToken, token, function (tk) { return new annotations.InjectAnnotationBuilder(tk); }, config.defaultAnnoBuilder || annotations.AnnotationBuilderToken);
        }
        if (builder) {
            builder.container = container;
        }
        return builder;
    };
    ModuleBuilder.classAnnations = { "name": "ModuleBuilder", "params": { "constructor": [], "getPools": [], "getInjectedModule": ["type"], "getMetaAccessor": ["container", "token", "config"], "load": ["token", "config", "options"], "bootstrap": ["token", "config", "options"], "vaildParams": ["token", "config", "options"], "loadViaToken": ["token", "parent"], "loadViaConfig": ["config", "parent"], "import": ["token", "parent"], "getParentContainer": ["env"], "getAnnoBuilder": ["container", "token", "config"] } };
    tslib_1.__decorate([
        core_1.Inject(utils.ContainerPoolToken),
        tslib_1.__metadata("design:type", utils.ContainerPool)
    ], ModuleBuilder.prototype, "pools", void 0);
    ModuleBuilder = tslib_1.__decorate([
        core_1.Singleton(IModuleBuilder.ModuleBuilderToken),
        tslib_1.__metadata("design:paramtypes", [])
    ], ModuleBuilder);
    return ModuleBuilder;
}());
exports.ModuleBuilder = ModuleBuilder;


});

unwrapExports(ModuleBuilder_1);
var ModuleBuilder_2 = ModuleBuilder_1.InjectModuleLoadToken;
var ModuleBuilder_3 = ModuleBuilder_1.ModuleBuilder;

var modules = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(DIModuleInjector_1, exports);
tslib_1.__exportStar(DIModuleValidate, exports);
tslib_1.__exportStar(InjectedModule_1, exports);
tslib_1.__exportStar(IModuleBuilder, exports);
tslib_1.__exportStar(ModuleBuilder_1, exports);


});

unwrapExports(modules);

var BootModule_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });








/**
 * Bootstrap ext for ioc. auto run setup after registered.
 * with @IocExt('setup') decorator.
 * @export
 * @class BootModule
 */
var BootModule = /** @class */ (function () {
    function BootModule(container) {
        this.container = container;
    }
    /**
     * register aop for container.
     *
     * @memberof AopModule
     */
    BootModule.prototype.setup = function () {
        var container = this.container;
        var lifeScope = container.get(core_1.LifeScopeToken);
        lifeScope.registerDecorator(Annotation.Annotation, core_1.CoreActions.bindProvider, core_1.CoreActions.cache, core_1.CoreActions.componentBeforeInit, core_1.CoreActions.componentInit, core_1.CoreActions.componentAfterInit);
        lifeScope.registerDecorator(DIModule.DIModule, core_1.CoreActions.bindProvider, core_1.CoreActions.cache, core_1.CoreActions.componentBeforeInit, core_1.CoreActions.componentInit, core_1.CoreActions.componentAfterInit);
        lifeScope.registerDecorator(Bootstrap.Bootstrap, core_1.CoreActions.bindProvider, core_1.CoreActions.cache, core_1.CoreActions.componentBeforeInit, core_1.CoreActions.componentInit, core_1.CoreActions.componentAfterInit);
        container.use(annotations, modules, boot);
    };
    BootModule.classAnnations = { "name": "BootModule", "params": { "constructor": ["container"], "setup": [] } };
    BootModule = tslib_1.__decorate([
        core_1.IocExt('setup'),
        tslib_1.__param(0, core_1.Inject(core_1.ContainerToken)),
        tslib_1.__metadata("design:paramtypes", [Object])
    ], BootModule);
    return BootModule;
}());
exports.BootModule = BootModule;


});

unwrapExports(BootModule_1);
var BootModule_2 = BootModule_1.BootModule;

var RunnableBuilder_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });







/**
 * runnable events
 *
 * @export
 * @enum {number}
 */
var RunnableEvents;
(function (RunnableEvents) {
    /**
     * on root container created.
     */
    RunnableEvents["onRootContainerCreated"] = "onRootContainerCreated";
    /**
     * on root container inited.
     */
    RunnableEvents["onRootContainerInited"] = "onRootContainerInited";
    /**
     * on module created.
     */
    RunnableEvents["onModuleCreated"] = "onModuleCreated";
    /**
     * on registered runner use module.
     */
    RunnableEvents["registeredExt"] = "registeredExt";
})(RunnableEvents = exports.RunnableEvents || (exports.RunnableEvents = {}));
/**
 * runnable builder.
 *
 * @export
 * @class RunnableBuilder
 * @extends {ModuleBuilder}
 * @template T
 */
var RunnableBuilder = /** @class */ (function (_super) {
    tslib_1.__extends(RunnableBuilder, _super);
    function RunnableBuilder(baseURL) {
        var _this = _super.call(this) || this;
        _this.inited = false;
        _this._baseURL = baseURL;
        _this.customRegs = [];
        _this.globalModules = [];
        _this.beforeInitPds = new Map();
        _this.afterInitPds = new Map();
        _this.events = new utils.Events();
        _this.initEvents();
        return _this;
    }
    RunnableBuilder.prototype.initEvents = function () {
        var _this = this;
        this.on(RunnableEvents.onRootContainerInited, function (container) {
            _this.afterInitPds.forEach(function (val, key) {
                container.bindProvider(key, val);
            });
        });
    };
    RunnableBuilder.prototype.on = function (name, event) {
        this.events.on(name, event);
        return this;
    };
    RunnableBuilder.prototype.off = function (name, event) {
        this.events.off(name, event);
        return this;
    };
    RunnableBuilder.prototype.emit = function (name) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var _a;
        (_a = this.events).emit.apply(_a, [name].concat(args));
    };
    RunnableBuilder.prototype.getPools = function () {
        if (!this.pools) {
            this.pools = new utils.ContainerPool(this.createContainerBuilder());
            this.createDefaultContainer();
        }
        return this.pools;
    };
    RunnableBuilder.prototype.getRunRoot = function (container) {
        return this._baseURL || container.get(IRunnableBuilder.ProcessRunRootToken);
    };
    RunnableBuilder.prototype.createContainerBuilder = function () {
        return new core_1.ContainerBuilder();
    };
    /**
     * use module as global Depdences module.
     *
     * @param {...LoadType[]} modules
     * @returns {this}
     * @memberof RunnableBuilder
     */
    RunnableBuilder.prototype.use = function () {
        var modules$$1 = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            modules$$1[_i] = arguments[_i];
        }
        this.globalModules = this.globalModules.concat(modules$$1);
        this.inited = false;
        return this;
    };
    /**
     * bind provider
     *
     * @template T
     * @param {Token<T>} provide
     * @param {Token<T> | Factory<T>} provider
     * @param {boolean} [beforeInit]
     * @returns {this}
     * @memberof RunnableBuilder
     */
    RunnableBuilder.prototype.provider = function (provide, provider, beforeInit) {
        if (beforeInit) {
            this.beforeInitPds.set(provide, provider);
        }
        else {
            this.afterInitPds.set(provide, provider);
        }
        return this;
    };
    RunnableBuilder.prototype.getProvider = function (provide, beforeInit) {
        if (beforeInit) {
            return this.beforeInitPds.has(provide) ? this.beforeInitPds.get(provide) : null;
        }
        else {
            return this.afterInitPds.has(provide) ? this.afterInitPds.get(provide) : null;
        }
    };
    RunnableBuilder.prototype.load = function (token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.initRootContainer()];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, _super.prototype.load.call(this, token, config, options)];
                    case 2: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    RunnableBuilder.prototype.bootstrap = function (token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var params, injmdl, _a, builder, _b;
            return tslib_1.__generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        params = this.vaildParams(token, config, options);
                        options = params.options || {};
                        if (!params.token) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.load(params.token, params.config, options)];
                    case 1:
                        _a = _c.sent();
                        return [3 /*break*/, 4];
                    case 2: return [4 /*yield*/, this.load(params.config, options)];
                    case 3:
                        _a = _c.sent();
                        _c.label = 4;
                    case 4:
                        injmdl = _a;
                        options.env = injmdl;
                        builder = this.getBuilder(injmdl);
                        if (!params.token) return [3 /*break*/, 6];
                        return [4 /*yield*/, builder.bootstrap(params.token, params.config, options)];
                    case 5:
                        _b = _c.sent();
                        return [3 /*break*/, 8];
                    case 6: return [4 /*yield*/, builder.bootstrap(params.config, options)];
                    case 7:
                        _b = _c.sent();
                        _c.label = 8;
                    case 8: return [2 /*return*/, _b];
                }
            });
        });
    };
    /**
     * get module builder
     *
     * @param {(Token<T> | ModuleConfig<T>)} token
     * @param {ModuleEnv} [env]
     * @returns {IModuleBuilder<T>}
     * @memberof RunnableBuilder
     */
    RunnableBuilder.prototype.getBuilderByConfig = function (token, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var injmdl;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.load(token, options)];
                    case 1:
                        injmdl = _a.sent();
                        return [2 /*return*/, this.getBuilder(injmdl)];
                }
            });
        });
    };
    RunnableBuilder.prototype.getBuilder = function (injmdl) {
        var cfg = injmdl.config;
        var container = injmdl.container;
        var builder;
        if (cfg) {
            if (core_1.isClass(cfg.builder)) {
                if (!container.has(cfg.builder)) {
                    container.register(cfg.builder);
                }
            }
            if (core_1.isToken(cfg.builder)) {
                builder = container.resolve(cfg.builder);
            }
            else if (cfg.builder instanceof modules.ModuleBuilder) {
                builder = cfg.builder;
            }
        }
        var tko = injmdl.token;
        if (!builder && tko) {
            builder = container.getService(modules.ModuleBuilderToken, tko, function (tk) { return new modules.InjectModuleBuilderToken(tk); }, cfg.defaultBuilder || modules.ModuleBuilderToken);
        }
        return builder || this;
    };
    RunnableBuilder.prototype.getConfigManager = function () {
        if (!this.configMgr) {
            this.configMgr = this.createConfigureMgr();
        }
        return this.configMgr;
    };
    RunnableBuilder.prototype.createConfigureMgr = function () {
        var container = this.getPools().getDefault();
        return container.getService(IConfigureManager.ConfigureMgrToken, core_1.lang.getClass(this), { baseURL: this.getRunRoot(container) });
    };
    RunnableBuilder.prototype.createDefaultContainer = function () {
        var _this = this;
        var container = this.pools.getDefault();
        container.register(BootModule_1.BootModule);
        var chain = container.getBuilder().getInjectorChain(container);
        chain.first(container.resolve(modules.DIModuleInjectorToken));
        container.bindProvider(utils.ContainerPoolToken, function () { return _this.getPools(); });
        this.beforeInitPds.forEach(function (val, key) {
            container.bindProvider(key, val);
        });
        this.events.emit(RunnableEvents.onRootContainerCreated, container);
        return container;
    };
    RunnableBuilder.prototype.initRootContainer = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var container, configManager, config;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.inited) {
                            return [2 /*return*/];
                        }
                        container = this.getPools().getDefault();
                        return [4 /*yield*/, this.registerExts(container)];
                    case 1:
                        _a.sent();
                        configManager = this.getConfigManager();
                        return [4 /*yield*/, configManager.getConfig()];
                    case 2:
                        config = _a.sent();
                        return [4 /*yield*/, this.registerByConfigure(container, config)];
                    case 3:
                        _a.sent();
                        return [4 /*yield*/, configManager.bindBuilder(this)];
                    case 4:
                        _a.sent();
                        this.inited = true;
                        this.events.emit(RunnableEvents.onRootContainerInited, container);
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * register ioc exts
     *
     * @protected
     * @param {IContainer} container
     * @param {AppConfigure} config
     * @memberof RunnableBuilder
     */
    RunnableBuilder.prototype.registerExts = function (container) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var usedModules, types;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.globalModules.length) return [3 /*break*/, 2];
                        usedModules = this.globalModules;
                        return [4 /*yield*/, container.loadModule.apply(container, usedModules)];
                    case 1:
                        types = _a.sent();
                        this.emit(RunnableEvents.registeredExt, types, container);
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * register by configure.
     *
     * @protected
     * @param {IContainer} container
     * @param {ModuleConfig<T>} config
     * @returns {Promise<void>}
     * @memberof RunnableBuilder
     */
    RunnableBuilder.prototype.registerByConfigure = function (container, config) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        config.baseURL = this.getRunRoot(container);
                        return [4 /*yield*/, core_1.PromiseUtil.step(this.customRegs.map(function (cs) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                var tokens;
                                return tslib_1.__generator(this, function (_a) {
                                    switch (_a.label) {
                                        case 0: return [4 /*yield*/, cs(container, config, this)];
                                        case 1:
                                            tokens = _a.sent();
                                            return [2 /*return*/, tokens];
                                    }
                                });
                            }); }))];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    RunnableBuilder.classAnnations = { "name": "RunnableBuilder", "params": { "constructor": ["baseURL"], "initEvents": [], "on": ["name", "event"], "off": ["name", "event"], "emit": ["name", "args"], "getPools": [], "getRunRoot": ["container"], "createContainerBuilder": [], "use": ["modules"], "provider": ["provide", "provider", "beforeInit"], "getProvider": ["provide", "beforeInit"], "load": ["token", "config", "options"], "bootstrap": ["token", "config", "options"], "getBuilderByConfig": ["token", "options"], "getBuilder": ["injmdl"], "getConfigManager": [], "createConfigureMgr": [], "createDefaultContainer": [], "initRootContainer": [], "registerExts": ["container"], "registerByConfigure": ["container", "config"] } };
    RunnableBuilder = tslib_1.__decorate([
        core_1.Injectable(IRunnableBuilder.RunnableBuilderToken),
        tslib_1.__metadata("design:paramtypes", [String])
    ], RunnableBuilder);
    return RunnableBuilder;
}(modules.ModuleBuilder));
exports.RunnableBuilder = RunnableBuilder;


});

unwrapExports(RunnableBuilder_1);
var RunnableBuilder_2 = RunnableBuilder_1.RunnableEvents;
var RunnableBuilder_3 = RunnableBuilder_1.RunnableBuilder;

var IApplicationBuilder = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 *  application builder token.
 */
exports.ApplicationBuilderToken = new core_1.InjectToken('DI_AppBuilder');


});

unwrapExports(IApplicationBuilder);
var IApplicationBuilder_1 = IApplicationBuilder.ApplicationBuilderToken;

var AppConfigure = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 * application configuration token.
 */
exports.AppConfigureToken = new core_1.InjectToken('DI_APP_Configuration');


});

unwrapExports(AppConfigure);
var AppConfigure_1 = AppConfigure.AppConfigureToken;

var ApplicationBuilder_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });




/**
 * application events
 *
 * @export
 * @enum {number}
 */
exports.AppEvents = RunnableBuilder_1.RunnableEvents;
/**
 * application events
 */
exports.ApplicationEvents = RunnableBuilder_1.RunnableEvents;
/**
 * application builder.
 *
 * @export
 * @class Default ApplicationBuilder
 * @extends {ModuleBuilder}
 * @template T
 */
var ApplicationBuilder = /** @class */ (function (_super) {
    tslib_1.__extends(ApplicationBuilder, _super);
    function ApplicationBuilder(baseURL) {
        var _this = _super.call(this, baseURL) || this;
        _this.configs = [];
        return _this;
    }
    /**
     * use configuration.
     *
     * @param {(string | AppConfigure)} [config]
     * @returns {this} global config for this application.
     * @memberof Bootstrap
     */
    ApplicationBuilder.prototype.useConfiguration = function (config) {
        if (this.configMgr) {
            this.configMgr.useConfiguration(config);
        }
        this.configs.push(config);
        return this;
    };
    ApplicationBuilder.prototype.createConfigureMgr = function () {
        var cfgMgr = _super.prototype.createConfigureMgr.call(this);
        if (this.configs && this.configs.length) {
            this.configs.forEach(function (cfg) { return cfgMgr.useConfiguration(cfg); });
        }
        return cfgMgr;
    };
    /**
     * register by configure.
     *
     * @protected
     * @param {IContainer} container
     * @param {AppConfigure} config
     * @returns {Promise<void>}
     * @memberof RunnableBuilder
     */
    ApplicationBuilder.prototype.registerByConfigure = function (container, config) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, _super.prototype.registerByConfigure.call(this, container, config)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    ApplicationBuilder.classAnnations = { "name": "ApplicationBuilder", "params": { "constructor": ["baseURL"], "useConfiguration": ["config"], "createConfigureMgr": [], "registerByConfigure": ["container", "config"] } };
    ApplicationBuilder = tslib_1.__decorate([
        core_1.Injectable(IApplicationBuilder.ApplicationBuilderToken),
        tslib_1.__metadata("design:paramtypes", [String])
    ], ApplicationBuilder);
    return ApplicationBuilder;
}(RunnableBuilder_1.RunnableBuilder));
exports.ApplicationBuilder = ApplicationBuilder;


});

unwrapExports(ApplicationBuilder_1);
var ApplicationBuilder_2 = ApplicationBuilder_1.AppEvents;
var ApplicationBuilder_3 = ApplicationBuilder_1.ApplicationEvents;
var ApplicationBuilder_4 = ApplicationBuilder_1.ApplicationBuilder;

var boot = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(IConfigureManager, exports);
tslib_1.__exportStar(ConfigureManager_1, exports);
tslib_1.__exportStar(IRunnableBuilder, exports);
tslib_1.__exportStar(RunnableBuilder_1, exports);
tslib_1.__exportStar(IApplicationBuilder, exports);
tslib_1.__exportStar(AppConfigure, exports);
tslib_1.__exportStar(ApplicationBuilder_1, exports);


});

unwrapExports(boot);

var Bootstrap = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });



/**
 * create bootstrap decorator.
 *
 * @export
 * @template T
 * @param {string} name
 * @param {Token<IRunnableBuilder<any>>>} [builder] default builder
 * @param {Token<IAnnotationBuilder<any>>} [defaultAnnoBuilder] default type builder.
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IBootstrapDecorator<T>}
 */
function createBootstrapDecorator(name, defaultBuilder, defaultAnnoBuilder, adapter, metadataExtends) {
    return DIModule.createDIModuleDecorator(name, defaultBuilder, defaultAnnoBuilder, adapter, function (metadata) {
        if (metadataExtends) {
            metadataExtends(metadata);
        }
        if (metadata.builder) {
            setTimeout(function () {
                var _a;
                (_a = new boot.ApplicationBuilder()).use.apply(_a, (metadata.globals || [])).bootstrap(metadata);
            }, 300);
        }
        return metadata;
    });
}
exports.createBootstrapDecorator = createBootstrapDecorator;
/**
 * Bootstrap Decorator, definde class as mvc bootstrap module.
 *
 * @Bootstrap
 */
exports.Bootstrap = createBootstrapDecorator('Bootstrap', boot.ApplicationBuilderToken, IAnnotationBuilder.AnnotationBuilderToken);


});

unwrapExports(Bootstrap);
var Bootstrap_1 = Bootstrap.createBootstrapDecorator;
var Bootstrap_2 = Bootstrap.Bootstrap;

var decorators = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(Annotation, exports);
tslib_1.__exportStar(DIModule, exports);
tslib_1.__exportStar(Bootstrap, exports);


});

unwrapExports(decorators);

var D__Workspace_github_node_tsioc_packages_bootstrap_lib = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(decorators, exports);
tslib_1.__exportStar(boot, exports);
tslib_1.__exportStar(annotations, exports);
tslib_1.__exportStar(modules, exports);
tslib_1.__exportStar(runnable, exports);
tslib_1.__exportStar(utils, exports);
tslib_1.__exportStar(BootModule_1, exports);


});

var index$6 = unwrapExports(D__Workspace_github_node_tsioc_packages_bootstrap_lib);

return index$6;

})));

//# sourceMappingURL=sourcemaps/bootstrap.umd.js.map
