(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('tslib'), require('@ts-ioc/core'), require('reflect-metadata')) :
	typeof define === 'function' && define.amd ? define(['tslib', '@ts-ioc/core', 'reflect-metadata'], factory) :
	(global.bootstrap = global.bootstrap || {}, global.bootstrap.js = factory(global.tslib,global['@ts-ioc/core'],global.Reflect));
}(this, (function (tslib_1,core_1,reflectMetadata) { 'use strict';

tslib_1 = tslib_1 && tslib_1.hasOwnProperty('default') ? tslib_1['default'] : tslib_1;
core_1 = core_1 && core_1.hasOwnProperty('default') ? core_1['default'] : core_1;
reflectMetadata = reflectMetadata && reflectMetadata.hasOwnProperty('default') ? reflectMetadata['default'] : reflectMetadata;

function unwrapExports (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var IAnnotationBuilder = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

const annoBuilderDesc = 'DI_AnnotationBuilder';
/**
 * inject Annotation class builder.
 *
 * @export
 * @class InjectBootstrapBuilder
 * @extends {Registration<T>}
 * @template T
 */
class InjectAnnotationBuilder extends core_1.RefRegistration {
    constructor(type) {
        super(type, annoBuilderDesc);
    }
}
InjectAnnotationBuilder.classAnnations = { "name": "InjectAnnotationBuilder", "params": { "constructor": ["type"] } };
exports.InjectAnnotationBuilder = InjectAnnotationBuilder;
/**
 * Annotation class builder token.
 */
exports.AnnotationBuilderToken = new InjectAnnotationBuilder('');
/**
 * Default Annotation class builder token.
 */
exports.DefaultAnnotationBuilderToken = new InjectAnnotationBuilder('default');


});

unwrapExports(IAnnotationBuilder);
var IAnnotationBuilder_1 = IAnnotationBuilder.InjectAnnotationBuilder;
var IAnnotationBuilder_2 = IAnnotationBuilder.AnnotationBuilderToken;
var IAnnotationBuilder_3 = IAnnotationBuilder.DefaultAnnotationBuilderToken;

var Annotation = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * create type builder decorator
 *
 * @export
 * @template T
 * @param {string} name
 * @param {string} [decorType]
 * @param {(Token<IAnnotationBuilder<any>> | IAnnotationBuilder<any>)} [defaultBuilder]
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IAnnotationDecorator<T>}
 */
function createAnnotationDecorator(name, defaultBuilder, adapter, metadataExtends) {
    return core_1.createClassDecorator(name, args => {
        if (adapter) {
            adapter(args);
        }
    }, metadata => {
        if (metadataExtends) {
            metadata = metadataExtends(metadata);
        }
        if (defaultBuilder && !metadata.defaultAnnoBuilder) {
            metadata.defaultAnnoBuilder = defaultBuilder;
        }
        return metadata;
    });
}
exports.createAnnotationDecorator = createAnnotationDecorator;
/**
 * Annotation decorator, use to define class build way via config.
 *
 * @Annotation
 */
exports.Annotation = createAnnotationDecorator('Annotation', IAnnotationBuilder.AnnotationBuilderToken);


});

unwrapExports(Annotation);
var Annotation_1 = Annotation.createAnnotationDecorator;
var Annotation_2 = Annotation.Annotation;

var IModuleBuilder = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

const moduleBuilderDesc = 'DI_ModuleBuilder';
/**
 * inject module builder token.
 *
 * @export
 * @class InjectModuleBuilder
 * @extends {Registration<T>}
 * @template T
 */
class InjectModuleBuilderToken extends core_1.RefRegistration {
    constructor(type) {
        super(type, moduleBuilderDesc);
    }
}
InjectModuleBuilderToken.classAnnations = { "name": "InjectModuleBuilderToken", "params": { "constructor": ["type"] } };
exports.InjectModuleBuilderToken = InjectModuleBuilderToken;
/**
 * default module builder token.
 */
exports.ModuleBuilderToken = new InjectModuleBuilderToken(Object);


});

unwrapExports(IModuleBuilder);
var IModuleBuilder_1 = IModuleBuilder.InjectModuleBuilderToken;
var IModuleBuilder_2 = IModuleBuilder.ModuleBuilderToken;

var DIModule = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });



/**
 * create bootstrap decorator.
 *
 * @export
 * @template T
 * @param {string} name decorator name.
 * @param {Token<IModuleBuilder>} [defaultBuilder]
 * @param {Token<IAnnotationBuilder<any>>} [defaultAnnoBuilder]
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IDIModuleDecorator<T>}
 */
function createDIModuleDecorator(name, defaultBuilder, defaultAnnoBuilder, adapter, metadataExtends) {
    return core_1.createClassDecorator(name, args => {
        if (adapter) {
            adapter(args);
        }
    }, metadata => {
        if (metadataExtends) {
            metadata = metadataExtends(metadata);
        }
        if (!metadata.name && core_1.isClass(metadata.token)) {
            let isuglify = /^[a-z]$/.test(metadata.token.name);
            if (isuglify && metadata.token.classAnnations) {
                metadata.name = metadata.token.classAnnations.name;
            }
            else {
                metadata.name = metadata.token.name;
            }
        }
        metadata.decorType = name;
        if (defaultBuilder && !metadata.defaultBuilder) {
            metadata.defaultBuilder = defaultBuilder;
        }
        if (defaultAnnoBuilder && !metadata.defaultAnnoBuilder) {
            metadata.defaultAnnoBuilder = defaultAnnoBuilder;
        }
        return metadata;
    });
}
exports.createDIModuleDecorator = createDIModuleDecorator;
/**
 * DIModule Decorator, definde class as DI module.
 *
 * @DIModule
 */
exports.DIModule = createDIModuleDecorator('DIModule', IModuleBuilder.ModuleBuilderToken, IAnnotationBuilder.AnnotationBuilderToken);


});

unwrapExports(DIModule);
var DIModule_1 = DIModule.createDIModuleDecorator;
var DIModule_2 = DIModule.DIModule;

var IConfigureManager = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 * configure manager token.
 */
exports.ConfigureMgrToken = new core_1.InjectToken('config-mgr');
/**
 * application default configuration token.
 */
exports.DefaultConfigureToken = new core_1.InjectToken('DI_Default_Configuration');
/**
 * configure loader token.
 */
exports.ConfigureLoaderToken = new core_1.InjectToken('DI_Configure_Loader');


});

unwrapExports(IConfigureManager);
var IConfigureManager_1 = IConfigureManager.ConfigureMgrToken;
var IConfigureManager_2 = IConfigureManager.DefaultConfigureToken;
var IConfigureManager_3 = IConfigureManager.ConfigureLoaderToken;

var ConfigureManager_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });



/**
 * configure manager.
 *
 * @export
 * @class ConfigureManager
 */
// @RefTo(RunnableBuilderToken)
let ConfigureManager = class ConfigureManager {
    /**
     * Creates an instance of ConfigureManager.
     * @param {string} [baseURL]
     * @memberof ConfigureManager
     */
    constructor(baseURL) {
        this.baseURL = baseURL;
        this.configs = [];
    }
    /**
     * use configuration.
     *
     * @param {(string | AppConfigure)} [config]
     * @returns {this} this configure manager.
     * @memberof Bootstrap
     */
    useConfiguration(config) {
        if (core_1.isUndefined(config)) {
            config = '';
        }
        // clean cached config.
        this.config = null;
        let idx = this.configs.indexOf(config);
        if (idx >= 0) {
            this.configs.splice(idx, 1);
        }
        this.configs.push(config);
        return this;
    }
    /**
     * bind runnable builder.
     *
     * @param {IRunnableBuilder<any>} builder
     * @memberof ConfigureManager
     */
    async bindBuilder(builder) {
        // let config = await this.getConfig();
    }
    /**
     * get config.
     *
     * @returns {Promise<T>}
     * @memberof ConfigureManager
     */
    async getConfig() {
        if (!this.config) {
            this.config = await this.initConfig();
        }
        return this.config;
    }
    /**
     * init config.
     *
     * @protected
     * @returns
     * @memberof ConfigureManager
     */
    async initConfig() {
        let config = await this.getDefaultConfig();
        if (this.configs.length < 1) {
            this.configs.push(''); // load default loader config.
        }
        let exts = await Promise.all(this.configs.map(cfg => {
            if (core_1.isString(cfg)) {
                return this.loadConfig(cfg);
            }
            else {
                return cfg;
            }
        }));
        exts.forEach(exCfg => {
            if (exCfg) {
                core_1.lang.assign(config, exCfg);
            }
        });
        return config;
    }
    /**
     * load config.
     *
     * @protected
     * @param {string} src
     * @returns {Promise<T>}
     * @memberof ConfigureManager
     */
    async loadConfig(src) {
        if (this.container.has(IConfigureManager.ConfigureLoaderToken)) {
            let loader = this.container.resolve(IConfigureManager.ConfigureLoaderToken, { baseURL: this.baseURL, container: this.container });
            return await loader.load(src);
        }
        else if (src) {
            let builder = this.container.getBuilder();
            let cfg = await builder.loader.load([src]);
            return cfg.length ? cfg[0] : null;
        }
        else {
            return null;
        }
    }
    /**
     * get default config.
     *
     * @protected
     * @returns {Promise<T>}
     * @memberof ConfigureManager
     */
    async getDefaultConfig() {
        if (this.container.has(IConfigureManager.DefaultConfigureToken)) {
            return this.container.resolve(IConfigureManager.DefaultConfigureToken);
        }
        else {
            return {};
        }
    }
};
ConfigureManager.classAnnations = { "name": "ConfigureManager", "params": { "constructor": ["baseURL"], "useConfiguration": ["config"], "bindBuilder": ["builder"], "getConfig": [], "initConfig": [], "loadConfig": ["src"], "getDefaultConfig": [] } };
tslib_1.__decorate([
    core_1.Inject(core_1.ContainerToken),
    tslib_1.__metadata("design:type", Object)
], ConfigureManager.prototype, "container", void 0);
ConfigureManager = tslib_1.__decorate([
    core_1.Injectable(IConfigureManager.ConfigureMgrToken),
    tslib_1.__metadata("design:paramtypes", [String])
], ConfigureManager);
exports.ConfigureManager = ConfigureManager;


});

unwrapExports(ConfigureManager_1);
var ConfigureManager_2 = ConfigureManager_1.ConfigureManager;

var IRunnableBuilder = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 *  process run root.
 */
exports.ProcessRunRootToken = new core_1.InjectToken('ioc_processRunRoot');
/**
 * runable builder token.
 */
exports.RunnableBuilderToken = new core_1.InjectToken('DI_RunnableBuilder');


});

unwrapExports(IRunnableBuilder);
var IRunnableBuilder_1 = IRunnableBuilder.ProcessRunRootToken;
var IRunnableBuilder_2 = IRunnableBuilder.RunnableBuilderToken;

var DIModuleValidate = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });



/**
 * DIModule Validate Token
 */
exports.DIModuleValidateToken = new core_1.InjectModuleValidateToken(DIModule.DIModule.toString());
/**
 * DIModuel Validate
 *
 * @export
 * @class DIModuelValidate
 * @extends {ModuelValidate}
 */
let DIModuelValidate = class DIModuelValidate extends core_1.ModuelValidate {
    getDecorator() {
        return DIModule.DIModule.toString();
    }
};
DIModuelValidate.classAnnations = { "name": "DIModuelValidate", "params": { "getDecorator": [] } };
DIModuelValidate = tslib_1.__decorate([
    core_1.Singleton(exports.DIModuleValidateToken)
], DIModuelValidate);
exports.DIModuelValidate = DIModuelValidate;


});

unwrapExports(DIModuleValidate);
var DIModuleValidate_1 = DIModuleValidate.DIModuleValidateToken;
var DIModuleValidate_2 = DIModuleValidate.DIModuelValidate;

var ContainerPool_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 * container pool
 *
 * @export
 * @class ContainerPool
 */
class ContainerPool {
    constructor(containerBuilder) {
        this.containerBuilder = containerBuilder;
        this.pools = new Map();
    }
    createContainer() {
        return this.containerBuilder.create();
    }
    getTokenKey(token) {
        if (token instanceof core_1.Registration) {
            return token.toString();
        }
        return token;
    }
    isDefault(container) {
        return container === this._default;
    }
    hasDefault() {
        return !!this._default;
    }
    getDefault() {
        if (!this._default) {
            this._default = this.createContainer();
        }
        return this._default;
    }
    set(token, container) {
        let key = this.getTokenKey(token);
        if (this.pools.has(key)) {
            console.log(`${token.toString()} module has loaded`);
        }
        this.pools.set(key, container);
    }
    get(token) {
        let key = this.getTokenKey(token);
        if (!this.has(key)) {
            return null;
        }
        return this.pools.get(token);
    }
    has(token) {
        return this.pools.has(this.getTokenKey(token));
    }
    create(parent) {
        parent = parent || this.getDefault();
        let container = parent.getBuilder().create();
        this.setParent(container, parent);
        return container;
    }
    setParent(container, parent) {
        if (this.isDefault(container)) {
            return;
        }
        if (parent && parent !== container) {
            container.parent = parent;
        }
        else {
            container.parent = this.getDefault();
        }
    }
}
ContainerPool.classAnnations = { "name": "ContainerPool", "params": { "constructor": ["containerBuilder"], "createContainer": [], "getTokenKey": ["token"], "isDefault": ["container"], "hasDefault": [], "getDefault": [], "set": ["token", "container"], "get": ["token"], "has": ["token"], "create": ["parent"], "setParent": ["container", "parent"] } };
exports.ContainerPool = ContainerPool;
/**
 *  container pool token.
 */
exports.ContainerPoolToken = new core_1.InjectToken('DI_ContainerPool');


});

unwrapExports(ContainerPool_1);
var ContainerPool_2 = ContainerPool_1.ContainerPool;
var ContainerPool_3 = ContainerPool_1.ContainerPoolToken;

var Events_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 * custom events.
 *
 * @export
 * @class Events
 */
class Events {
    constructor() {
        this.eventsMap = {};
    }
    on(name, event) {
        this.eventsMap[name] = this.eventsMap[name] || [];
        if (this.eventsMap[name].indexOf(event) < 0) {
            this.eventsMap[name].push(event);
        }
        return this;
    }
    once(name, event) {
        let onceEvent = (...args) => {
            event(...args);
            this.off(name, onceEvent);
        };
        this.on(name, onceEvent);
    }
    off(name, event) {
        if (this.eventsMap[name]) {
            if (event) {
                this.eventsMap[name].splice(this.eventsMap[name].indexOf(event), 1);
            }
            else {
                delete this.eventsMap[name];
            }
        }
        return this;
    }
    emit(name, ...args) {
        let events = this.eventsMap[name];
        if (core_1.isArray(events)) {
            events.forEach(ev => {
                ev(...args);
            });
        }
    }
}
Events.classAnnations = { "name": "Events", "params": { "constructor": [], "on": ["name", "event"], "once": ["name", "event"], "off": ["name", "event"], "emit": ["name", "args"] } };
exports.Events = Events;


});

unwrapExports(Events_1);
var Events_2 = Events_1.Events;

var types = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * is target di module class or not.
 *
 * @export
 * @param {*} target
 * @returns
 */
function isDIModuleClass(target) {
    return core_1.isClass(target) && core_1.hasOwnClassMetadata(DIModule.DIModule, target);
}
exports.isDIModuleClass = isDIModuleClass;


});

unwrapExports(types);
var types_1 = types.isDIModuleClass;

var utils = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(ContainerPool_1, exports);
tslib_1.__exportStar(Events_1, exports);
tslib_1.__exportStar(types, exports);


});

unwrapExports(utils);

var InjectedModule_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 * injected module.
 *
 * @export
 * @class InjectedModule
 * @template T
 */
class InjectedModule {
    constructor(token, config, container, type, exports) {
        this.token = token;
        this.config = config;
        this.container = container;
        this.type = type;
        this.exports = exports;
    }
    getProviderMap() {
        if (!this._map) {
            this._map = this.container.resolveValue(new core_1.InjectReference(core_1.ProviderMap, this.type || this.token));
        }
        return this._map;
    }
    resolve(token, ...providers) {
        let pdr = this.getProviderMap();
        if (pdr && pdr.hasRegister(token)) {
            return pdr.resolve(token, ...providers);
        }
        else {
            return this.container.resolveValue(token, ...providers);
        }
    }
    hasRegister(key) {
        if (this.container.hasRegister(key)) {
            return true;
        }
        else {
            let pdr = this.getProviderMap();
            return pdr && pdr.hasRegister(key);
        }
    }
}
InjectedModule.classAnnations = { "name": "InjectedModule", "params": { "constructor": ["token", "config", "container", "type", "exports"], "getProviderMap": [], "resolve": ["token", "providers"], "hasRegister": ["key"] } };
exports.InjectedModule = InjectedModule;
/**
 * Injected Module Token.
 *
 * @export
 * @class InjectModuleMetaConfigToken
 * @extends {Registration<Type<T>>}
 * @template T
 */
class InjectedModuleToken extends core_1.Registration {
    constructor(type) {
        super(type, 'InjectedModule');
    }
}
InjectedModuleToken.classAnnations = { "name": "InjectedModuleToken", "params": { "constructor": ["type"] } };
exports.InjectedModuleToken = InjectedModuleToken;


});

unwrapExports(InjectedModule_1);
var InjectedModule_2 = InjectedModule_1.InjectedModule;
var InjectedModule_3 = InjectedModule_1.InjectedModuleToken;

var DIModuleInjector_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });






/**
 * DIModule injector token.
 */
exports.DIModuleInjectorToken = new core_1.InjectModuleInjectorToken(DIModule.DIModule.toString());
/**
 * DIModule injector.
 *
 * @export
 * @class DIModuleInjector
 * @extends {ModuleInjector}
 */
let DIModuleInjector = class DIModuleInjector extends core_1.ModuleInjector {
    constructor(validate) {
        super(validate);
    }
    async setup(container, type) {
        await this.importModule(container, type);
    }
    syncSetup(container, type) {
        this.importModule(container, type);
    }
    valid(container, type) {
        if (!this.validate) {
            return true;
        }
        return this.validate.valid(type);
    }
    getMetaAccessor(container, token) {
        return container.getService(core_1.MetaAccessorToken, token);
    }
    async importModule(container, type) {
        let pools = container.get(utils.ContainerPoolToken);
        let newContainer = pools.create(container);
        newContainer.register(type);
        let builder = newContainer.getBuilder();
        let decorator = this.validate.getDecorator();
        let accor = this.getMetaAccessor(newContainer, decorator);
        let metaConfig = accor.getMetadata(type, newContainer, undefined, decorator ? dec => dec === decorator : undefined);
        metaConfig = await this.registerConfgureDepds(newContainer, metaConfig, type);
        let exps = [].concat(...builder.loader.getTypes(metaConfig.exports || []));
        let classProvides = [];
        exps.forEach(ty => {
            let classPd = newContainer.resolveValue(new core_1.InjectClassProvidesToken(ty));
            if (classPd && core_1.isArray(classPd.provides) && classPd.provides.length) {
                classProvides = classProvides.concat(classPd.provides);
            }
        });
        let injMd = new InjectedModule_1.InjectedModule(metaConfig.token || type, metaConfig, newContainer, type, exps);
        container.bindProvider(new InjectedModule_1.InjectedModuleToken(type), injMd);
        await this.importConfigExports(container, newContainer, injMd);
        return injMd;
    }
    async registerConfgureDepds(container, config, type) {
        if (core_1.isArray(config.imports) && config.imports.length) {
            await container.loadModule(...config.imports);
        }
        if (!type && core_1.isArray(config.providers) && config.providers.length) {
            await this.bindProvider(container, config.providers);
        }
        return config;
    }
    async importConfigExports(container, providerContainer, injMd) {
        if (container === providerContainer) {
            return container;
        }
        if (injMd) {
            let chain = container.getResolvers();
            chain.next(injMd);
            if (injMd.exports && injMd.exports.length) {
                providerContainer.getResolvers().toArray().filter(r => {
                    if (r instanceof core_1.Container) {
                        return false;
                    }
                    else {
                        return injMd.exports.indexOf(r.type) >= 0;
                    }
                }).forEach(r => {
                    chain.next(r);
                });
            }
        }
        return container;
    }
    bindProvider(container, providers) {
        let parser = container.getProviderParser();
        let pdrmap = parser.parse(...providers);
        let tokens = pdrmap.provides();
        tokens.forEach(key => {
            container.bindProvider(key, (...providers) => pdrmap.resolve(key, ...providers));
        });
        return tokens;
    }
};
DIModuleInjector.classAnnations = { "name": "DIModuleInjector", "params": { "constructor": ["validate"], "setup": ["container", "type"], "syncSetup": ["container", "type"], "valid": ["container", "type"], "getMetaAccessor": ["container", "token"], "importModule": ["container", "type"], "registerConfgureDepds": ["container", "config", "type"], "importConfigExports": ["container", "providerContainer", "injMd"], "bindProvider": ["container", "providers"] } };
DIModuleInjector = tslib_1.__decorate([
    core_1.Singleton(exports.DIModuleInjectorToken),
    tslib_1.__param(0, core_1.Inject(DIModuleValidate.DIModuleValidateToken)),
    tslib_1.__metadata("design:paramtypes", [Object])
], DIModuleInjector);
exports.DIModuleInjector = DIModuleInjector;


});

unwrapExports(DIModuleInjector_1);
var DIModuleInjector_2 = DIModuleInjector_1.DIModuleInjectorToken;
var DIModuleInjector_3 = DIModuleInjector_1.DIModuleInjector;

var AnnoBuildStrategy_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


class InjectAnnoBuildStrategyToken extends core_1.RefRegistration {
    constructor(type) {
        super(type, 'AnnoBuildStrategy');
    }
}
InjectAnnoBuildStrategyToken.classAnnations = { "name": "InjectAnnoBuildStrategyToken", "params": { "constructor": ["type"] } };
exports.InjectAnnoBuildStrategyToken = InjectAnnoBuildStrategyToken;
/**
 *  base AnnoBuildStrategy Token
 */
exports.AnnoBuildStrategyToken = new InjectAnnoBuildStrategyToken(Object);
/**
 * annotation instance build strategy.
 *
 * @export
 * @class AnnoBuildStrategy
 * @implements {IAnnoBuildStrategy<T>}
 * @template T
 */
let AnnoBuildStrategy = class AnnoBuildStrategy {
    async build(instance, config, options) {
        if (!instance) {
            return;
        }
        await this.beforeInit(instance, config);
        await this.process(instance, config, options);
        await this.afterInit(instance, config);
    }
    /**
     * after annotation instance init.
     *
     * @protected
     * @param {BootHooks<T>} instance
     * @param {AnnotationConfigure<T>} config
     * @returns {Promise<void>}
     * @memberof AnnoBuildStrategy
     */
    async beforeInit(instance, config) {
        if (core_1.isFunction(instance.anBeforeInit)) {
            await instance.anBeforeInit(config);
        }
    }
    /**
     * build annotation instance process.
     *
     * @protected
     * @param {T} instance
     * @param {AnnotationConfigure<T>} config
     * @param {BuildOptions<T>} [options]
     * @returns {Promise<void>}
     * @memberof AnnoBuildStrategy
     */
    async process(instance, config, options) {
    }
    /**
     * after annotation instance init.
     *
     * @protected
     * @param {BootHooks<T>} instance
     * @param {AnnotationConfigure<T>} config
     * @returns {Promise<void>}
     * @memberof AnnoBuildStrategy
     */
    async afterInit(instance, config) {
        if (core_1.isFunction(instance.anAfterInit)) {
            await instance.anAfterInit(config);
        }
    }
};
AnnoBuildStrategy.classAnnations = { "name": "AnnoBuildStrategy", "params": { "build": ["instance", "config", "options"], "beforeInit": ["instance", "config"], "process": ["instance", "config", "options"], "afterInit": ["instance", "config"] } };
AnnoBuildStrategy = tslib_1.__decorate([
    core_1.Singleton(exports.AnnoBuildStrategyToken)
], AnnoBuildStrategy);
exports.AnnoBuildStrategy = AnnoBuildStrategy;


});

unwrapExports(AnnoBuildStrategy_1);
var AnnoBuildStrategy_2 = AnnoBuildStrategy_1.InjectAnnoBuildStrategyToken;
var AnnoBuildStrategy_3 = AnnoBuildStrategy_1.AnnoBuildStrategyToken;
var AnnoBuildStrategy_4 = AnnoBuildStrategy_1.AnnoBuildStrategy;

var Runner_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 * boot element.
 *
 * @export
 * @abstract
 * @class Boot
 * @implements {IBoot}
 */
class Runner {
    constructor(token, instance, config) {
        this.token = token;
        this.instance = instance;
        this.config = config;
    }
    getTarget() {
        return this.instance;
    }
    getTargetToken() {
        return this.token || core_1.lang.getClass(this.instance);
    }
}
Runner.classAnnations = { "name": "Runner", "params": { "constructor": ["token", "instance", "config"], "getTarget": [], "getTargetToken": [], "run": ["data"] } };
exports.Runner = Runner;
/**
 * boot element
 *
 * @export
 * @class Boot
 * @extends {Runner<any>}
 */
class Boot extends Runner {
    constructor(token, instance, config) {
        super(token, instance, config);
        this.token = token;
        this.instance = instance;
        this.config = config;
    }
}
Boot.classAnnations = { "name": "Boot", "params": { "constructor": ["token", "instance", "config"] } };
exports.Boot = Boot;
/**
 * target is runner or not.
 *
 * @export
 * @param {*} target
 * @returns {target is IRunner<any>}
 */
function isRunner(target) {
    if (target instanceof Runner) {
        return true;
    }
    if (target && core_1.isFunction(target.run)) {
        return true;
    }
    return false;
}
exports.isRunner = isRunner;


});

unwrapExports(Runner_1);
var Runner_2 = Runner_1.Runner;
var Runner_3 = Runner_1.Boot;
var Runner_4 = Runner_1.isRunner;

var Service_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 * base service.
 *
 * @export
 * @abstract
 * @class Service
 * @implements {IService}
 */
class Service {
    constructor(token, instance, config) {
        this.token = token;
        this.instance = instance;
        this.config = config;
    }
    /**
     * get target.
     *
     * @returns {T}
     * @memberof Service
     */
    getTarget() {
        return this.instance;
    }
    getTargetToken() {
        return this.token || core_1.lang.getClass(this.instance);
    }
}
Service.classAnnations = { "name": "Service", "params": { "constructor": ["token", "instance", "config"], "getTarget": [], "getTargetToken": [], "start": ["data"], "stop": [] } };
exports.Service = Service;
/**
 * target is service or not.
 *
 * @export
 * @param {*} target
 * @returns {target is IService<any>}
 */
function isService(target) {
    if (target instanceof Service) {
        return true;
    }
    if (target && core_1.isFunction(target.start)) {
        return true;
    }
    return false;
}
exports.isService = isService;


});

unwrapExports(Service_1);
var Service_2 = Service_1.Service;
var Service_3 = Service_1.isService;

var Runnable = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 * application runner token.
 *
 * @export
 * @class InjectRunnerToken
 * @extends {Registration<IRunner<T>>}
 * @template T
 */
class InjectRunnableToken extends core_1.RefRegistration {
    constructor(type) {
        super(type, 'Runnable');
    }
}
InjectRunnableToken.classAnnations = { "name": "InjectRunnableToken", "params": { "constructor": ["type"] } };
exports.InjectRunnableToken = InjectRunnableToken;
/**
 * default service token.
 */
exports.ServiceToken = new InjectRunnableToken('service');
/**
 * default runner token.
 */
exports.RunnerToken = new InjectRunnableToken('runner');


});

unwrapExports(Runnable);
var Runnable_1 = Runnable.InjectRunnableToken;
var Runnable_2 = Runnable.ServiceToken;
var Runnable_3 = Runnable.RunnerToken;

var runnable = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(Runner_1, exports);
tslib_1.__exportStar(Service_1, exports);
tslib_1.__exportStar(Runnable, exports);


});

unwrapExports(runnable);

var AnnotationBuilder_2 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

var AnnotationBuilder_1;




/**
 * Annotation class builder. build class with metadata and config.
 *
 * @export
 * @class AnnotationBuilder
 * @implements {implements IAnnotationBuilder<T>}
 * @template T
 */
let AnnotationBuilder = AnnotationBuilder_1 = class AnnotationBuilder {
    constructor() {
    }
    /**
     * get metadata accessor.
     *
     * @param {(Token<any>|| AnnotationConfigure<T>)} token
     * @param {AnnotationConfigure<T>} [config]
     * @returns {IMetaAccessor<any>}
     * @memberof AnnotationBuilder
     */
    getMetaAccessor(token, config) {
        let mtk;
        if (core_1.isToken(token)) {
            mtk = token;
        }
        else {
            config = token;
        }
        return this.container.getService(core_1.MetaAccessorToken, mtk ? [mtk, core_1.lang.getClass(this)] : core_1.lang.getClass(this), tk => new core_1.InjectMetaAccessorToken(tk), config ? (config.defaultMetaAccessor || core_1.MetaAccessorToken) : core_1.MetaAccessorToken);
    }
    /**
     * build token type with config.
     *
     * @param {(Token<T> | AnnotationConfigure<T>)} token
     * @param {(AnnotationConfigure<T> | BuildOptions<T>)} [config]
     * @param {BuildOptions<T>} [options]
     * @returns {Promise<T>}
     * @memberof AnnotationBuilder
     */
    async build(token, config, options) {
        let tk;
        let cfg;
        if (core_1.isToken(token)) {
            tk = token;
            if (core_1.isUndefined(options)) {
                options = config;
                cfg = null;
            }
            else {
                cfg = config;
            }
            cfg = this.getMetaAccessor(tk, cfg).getMetadata(tk, this.container, cfg);
        }
        else {
            options = config;
            cfg = token;
            let accessor = this.getMetaAccessor(cfg);
            tk = accessor.getToken(cfg, this.container);
            if (core_1.isToken(tk)) {
                cfg = accessor.getMetadata(tk, this.container, cfg);
            }
        }
        if (core_1.isClass(tk) && !this.container.hasRegister(tk)) {
            this.container.register(tk);
        }
        if (options && core_1.isFunction(options.vaild) && !options.vaild(tk)) {
            return null;
        }
        let builder = (options && options.builder) ? options.builder : this.getBuilder(tk, cfg);
        if (!this.isEqual(builder)) {
            return await builder.build(tk, cfg, core_1.lang.assign(options || {}, { builder: builder }));
        }
        else {
            await this.registerExts(cfg);
            let instance = await this.createInstance(tk, cfg, options);
            if (!instance) {
                return null;
            }
            let strategy = this.container.getService(AnnoBuildStrategy_1.AnnoBuildStrategyToken, [
                tk || core_1.lang.getClass(instance),
                ...((options && options.target) ? [{ target: core_1.lang.getClass(options.target), level: core_1.RefTagLevel.self }] : []),
                { target: core_1.lang.getClass(this), level: core_1.RefTagLevel.self }
            ], tk => new AnnoBuildStrategy_1.InjectAnnoBuildStrategyToken(tk));
            if (strategy) {
                await strategy.build(instance, cfg, options);
            }
            if (options && core_1.isFunction(options.onCompleted)) {
                options.onCompleted(tk, cfg, instance, this);
            }
            return instance;
        }
    }
    /**
     * run runable.
     *
     * @param {(Token<T> | AnnotationConfigure<T>)} runable
     * @param {*} [data] bootstrap data, build data, Runnable data.
     * @returns {Promise<Runnable<T>>}
     * @memberof IGModuleBuilder
     */
    async boot(runable, config, options) {
        let instance;
        let builder;
        let tk;
        let cfg;
        let onCompleted = (token, config, hook, builder) => {
            tk = token;
            cfg = config;
            instance = hook;
            builder = builder;
        };
        if (core_1.isToken(runable)) {
            if (core_1.isUndefined(options)) {
                options = config;
                cfg = null;
            }
            else {
                cfg = config;
            }
            await this.build(runable, cfg, core_1.lang.assign({
                onCompleted: onCompleted
            }, options));
        }
        else {
            await this.build(runable, core_1.lang.assign({
                onCompleted: onCompleted
            }, config));
        }
        builder = builder || this;
        let runner = builder.resolveRunable(instance, cfg, tk, options);
        let data = options ? options.data : undefined;
        if (runnable.isRunner(runner)) {
            await runner.run(data);
        }
        else if (runnable.isService(runner)) {
            await runner.start(data);
        }
        return runner;
    }
    /**
     * run annotation instance.
     *
     * @param {T} instance
     * @param {AnnotationConfigure<T>} [config]
     * @param {Token<T>} [token]
     * @returns {Promise<Runnable<T>>}
     * @memberof AnnotationBuilder
     */
    resolveRunable(instance, config, token, options) {
        if (!instance) {
            return null;
        }
        let tk;
        if (core_1.isToken(token)) {
            tk = token;
        }
        else {
            options = token;
        }
        tk = tk || core_1.lang.getClass(instance);
        if (!config) {
            config = this.getMetaAccessor(tk).getMetadata(tk, this.container);
        }
        if (instance instanceof runnable.Runner || instance instanceof runnable.Service) {
            return instance;
        }
        else {
            let providers = [{ provide: tk, useValue: instance }, { token: tk, instance: instance, config: config }];
            return this.container.getService([runnable.RunnerToken, runnable.ServiceToken], [
                tk,
                ...((options && options.target) ? [{ target: core_1.lang.getClass(options.target), level: core_1.RefTagLevel.self }] : []),
                { target: core_1.lang.getClass(this), level: core_1.RefTagLevel.self }
            ], tk => new runnable.InjectRunnableToken(tk), config.defaultRunnable || true, ...providers);
        }
    }
    async createInstance(token, config, options) {
        if (!core_1.isToken(token)) {
            return null;
        }
        let instance = this.resolveToken(token, options ? options.target : undefined);
        if (core_1.isNullOrUndefined(instance)) {
            console.log(`can not find token ${token ? token.toString() : null} in container.`);
            return null;
        }
        return instance;
    }
    getBuilder(token, config, options) {
        let builder;
        if (config && config.annoBuilder) {
            if (core_1.isClass(config.annoBuilder)) {
                if (!this.container.has(config.annoBuilder)) {
                    this.container.register(config.annoBuilder);
                }
            }
            if (core_1.isToken(config.annoBuilder)) {
                builder = this.container.resolve(config.annoBuilder);
            }
            else if (config.annoBuilder instanceof AnnotationBuilder_1) {
                builder = config.annoBuilder;
            }
        }
        if (options && options.builder) {
            builder = options.builder;
        }
        if (!builder && token) {
            builder = this.container.getService(IAnnotationBuilder.AnnotationBuilderToken, [
                token,
                ...(options && options.target) ? [{ target: core_1.lang.getClass(options.target), level: core_1.RefTagLevel.self }] : []
            ], (tk) => new IAnnotationBuilder.InjectAnnotationBuilder(tk), config.defaultAnnoBuilder || IAnnotationBuilder.AnnotationBuilderToken);
        }
        if (builder && !builder.container) {
            builder.container = this.container;
        }
        return builder || this;
    }
    /**
     * register extension before create instance.
     *
     * @protected
     * @param {AnnotationConfigure<T>} [config]
     * @memberof AnnotationBuilder
     */
    async registerExts(config) {
    }
    isEqual(build) {
        if (!build) {
            return false;
        }
        if (build === this) {
            return true;
        }
        if (core_1.lang.getClass(build) === core_1.lang.getClass(this)) {
            return true;
        }
        return false;
    }
    resolveToken(token, target) {
        if (target) {
            return this.container.getService(token, target);
        }
        return this.container.resolve(token);
    }
};
AnnotationBuilder.classAnnations = { "name": "AnnotationBuilder", "params": { "constructor": [], "getMetaAccessor": ["token", "config"], "build": ["token", "config", "options"], "boot": ["runable", "config", "options"], "resolveRunable": ["instance", "config", "token", "options"], "createInstance": ["token", "config", "options"], "getBuilder": ["token", "config", "options"], "registerExts": ["config"], "isEqual": ["build"], "resolveToken": ["token", "target"] } };
tslib_1.__decorate([
    core_1.Inject(core_1.ContainerToken),
    tslib_1.__metadata("design:type", Object)
], AnnotationBuilder.prototype, "container", void 0);
AnnotationBuilder = AnnotationBuilder_1 = tslib_1.__decorate([
    core_1.Singleton(IAnnotationBuilder.AnnotationBuilderToken),
    tslib_1.__metadata("design:paramtypes", [])
], AnnotationBuilder);
exports.AnnotationBuilder = AnnotationBuilder;


});

unwrapExports(AnnotationBuilder_2);
var AnnotationBuilder_3 = AnnotationBuilder_2.AnnotationBuilder;

var annotations = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(IAnnotationBuilder, exports);
tslib_1.__exportStar(AnnoBuildStrategy_1, exports);
tslib_1.__exportStar(AnnotationBuilder_2, exports);


});

unwrapExports(annotations);

var ModuleBuilder_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });







/**
 * inject module load token.
 *
 * @export
 * @class InjectModuleLoadToken
 * @extends {Registration<T>}
 * @template T
 */
class InjectModuleLoadToken extends core_1.Registration {
    constructor(token) {
        super(token, 'module_loader');
    }
}
InjectModuleLoadToken.classAnnations = { "name": "InjectModuleLoadToken", "params": { "constructor": ["token"] } };
exports.InjectModuleLoadToken = InjectModuleLoadToken;
/**
 * module builder
 *
 * @export
 * @class ModuleBuilder
 * @implements {IModuleBuilder}
 * @template T
 */
let ModuleBuilder = class ModuleBuilder {
    constructor() {
    }
    getPools() {
        return this.pools;
    }
    getInjectedModule(type) {
        return this.getPools().getDefault().get(new InjectedModule_1.InjectedModuleToken(type));
    }
    /**
     * get metadata accessor.
     *
     * @param {IContainer} container
     * @param {Token<any>} token
     * @param {ModuleConfigure} [config]
     * @returns {IMetaAccessor<any>}
     * @memberof ModuleBuilder
     */
    getMetaAccessor(container, token, config) {
        let mtk;
        if (core_1.isToken(token)) {
            mtk = token;
        }
        else {
            config = token;
        }
        return container.getService(core_1.MetaAccessorToken, mtk ? [mtk, core_1.lang.getClass(this)] : core_1.lang.getClass(this), tk => new core_1.InjectMetaAccessorToken(tk), config ? (config.defaultMetaAccessor || core_1.MetaAccessorToken) : core_1.MetaAccessorToken);
    }
    /**
     * load module.
     *
     * @param {(Token<T> | ModuleConfigure)} token
     * @param {(ModuleConfig<T> | BootOptions<T>)} [config]
     * @param {BootOptions<T>} [options]
     * @returns {Promise<InjectedModule<T>>}
     * @memberof ModuleBuilder
     */
    async load(token, config, options) {
        let params = this.vaildParams(token, config, options);
        let env = params.options ? params.options.env : null;
        if (env instanceof InjectedModule_1.InjectedModule) {
            return env;
        }
        let parent = await this.getParentContainer(env);
        if (params.token) {
            let injmd = await this.loadViaToken(params.token, parent);
            if (params.config) {
                await this.loadViaConfig(params.config, injmd.container || parent);
            }
            return injmd;
        }
        else {
            return await this.loadViaConfig(params.config, parent);
        }
    }
    /**
     * bootstrap with module and config.
     *
     * @param {(Token<T> | ModuleConfig<T>)} token
     * @param {(ModuleConfig<T> | BootOptions<T>)} [config]
     * @param {BootOptions<T>} [options]
     * @returns {Promise<Runnable<T>>}
     * @memberof ModuleBuilder
     */
    async bootstrap(token, config, options) {
        let params = this.vaildParams(token, config, options);
        options = params.options || {};
        let injmdl;
        if (params.token) {
            injmdl = await this.load(params.token, params.config, options);
        }
        else {
            injmdl = await this.load(params.config, options);
        }
        options.env = injmdl;
        let cfg = injmdl.config;
        let tk = injmdl.token || injmdl.type;
        let container = injmdl.container;
        let accessor = this.getMetaAccessor(container, tk);
        let bootToken = accessor.getBootToken(cfg, container);
        if (bootToken) {
            let anBuilder = this.getAnnoBuilder(container, bootToken, cfg);
            return await anBuilder.boot(bootToken, cfg, options);
        }
        else {
            let mdBuilder = this.getAnnoBuilder(container, tk, cfg);
            return await mdBuilder.boot(tk, cfg, options);
        }
    }
    vaildParams(token, config, options) {
        let params = {};
        if (core_1.isToken(token)) {
            params.token = token;
            if (core_1.isUndefined(options)) {
                params.options = config;
            }
            else {
                params.config = config;
            }
        }
        else {
            params.config = token;
            params.options = config;
        }
        return params;
    }
    async loadViaToken(token, parent) {
        let injmdl = await this.import(token, parent);
        if (!injmdl) {
            let cfg = this.getMetaAccessor(parent, token).getMetadata(token, parent);
            injmdl = new InjectedModule_1.InjectedModule(token, cfg, parent);
        }
        return injmdl;
    }
    async loadViaConfig(config, parent) {
        let injmd = null;
        let token = this.getMetaAccessor(parent, config).getToken(config, parent);
        if (token) {
            injmd = await this.import(token, parent);
            if (!injmd) {
                let cfg = this.getMetaAccessor(parent, token).getMetadata(token, parent, config);
                injmd = new InjectedModule_1.InjectedModule(token, cfg, parent);
            }
        }
        else {
            token = null;
        }
        if (core_1.isArray(config.imports) && config.imports.length) {
            await parent.loadModule(...config.imports);
        }
        if (core_1.isArray(config.providers) && config.providers.length) {
            let parser = parent.getProviderParser();
            let pdrmap = parser.parse(...config.providers);
            pdrmap.keys().forEach(key => {
                core_1.isToken(key) && parent.bindProvider(key, (...providers) => pdrmap.resolve(key, ...providers));
            });
        }
        if (injmd) {
            injmd.config = core_1.lang.assign({}, injmd.config, config);
        }
        else {
            injmd = new InjectedModule_1.InjectedModule(token, config, parent);
        }
        return injmd;
    }
    async import(token, parent) {
        let type = core_1.isClass(token) ? token : parent.getTokenImpl(token);
        if (core_1.isClass(type)) {
            let key = new InjectedModule_1.InjectedModuleToken(type);
            if (parent.has(key)) {
                return parent.get(key);
            }
            else {
                await parent.loadModule(type);
                if (parent.has(key)) {
                    return parent.get(key);
                }
            }
        }
        return null;
    }
    async getParentContainer(env) {
        let parent;
        if (env) {
            if (env instanceof core_1.Container) {
                parent = env;
            }
            else if (env instanceof InjectedModule_1.InjectedModule) {
                parent = env.container.parent;
            }
        }
        if (!parent) {
            parent = this.getPools().getDefault();
        }
        return parent;
    }
    getAnnoBuilder(container, token, config) {
        let builder;
        if (core_1.isClass(config.annoBuilder)) {
            if (!container.has(config.annoBuilder)) {
                container.register(config.annoBuilder);
            }
        }
        if (core_1.isToken(config.annoBuilder)) {
            builder = container.resolve(config.annoBuilder);
        }
        else if (config.annoBuilder instanceof annotations.AnnotationBuilder) {
            builder = config.annoBuilder;
        }
        if (!builder && token) {
            builder = container.getService(annotations.AnnotationBuilderToken, token, tk => new annotations.InjectAnnotationBuilder(tk), config.defaultAnnoBuilder || annotations.AnnotationBuilderToken);
        }
        if (builder) {
            builder.container = container;
        }
        return builder;
    }
};
ModuleBuilder.classAnnations = { "name": "ModuleBuilder", "params": { "constructor": [], "getPools": [], "getInjectedModule": ["type"], "getMetaAccessor": ["container", "token", "config"], "load": ["token", "config", "options"], "bootstrap": ["token", "config", "options"], "vaildParams": ["token", "config", "options"], "loadViaToken": ["token", "parent"], "loadViaConfig": ["config", "parent"], "import": ["token", "parent"], "getParentContainer": ["env"], "getAnnoBuilder": ["container", "token", "config"] } };
tslib_1.__decorate([
    core_1.Inject(utils.ContainerPoolToken),
    tslib_1.__metadata("design:type", utils.ContainerPool)
], ModuleBuilder.prototype, "pools", void 0);
ModuleBuilder = tslib_1.__decorate([
    core_1.Singleton(IModuleBuilder.ModuleBuilderToken),
    tslib_1.__metadata("design:paramtypes", [])
], ModuleBuilder);
exports.ModuleBuilder = ModuleBuilder;


});

unwrapExports(ModuleBuilder_1);
var ModuleBuilder_2 = ModuleBuilder_1.InjectModuleLoadToken;
var ModuleBuilder_3 = ModuleBuilder_1.ModuleBuilder;

var modules = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(DIModuleInjector_1, exports);
tslib_1.__exportStar(DIModuleValidate, exports);
tslib_1.__exportStar(InjectedModule_1, exports);
tslib_1.__exportStar(IModuleBuilder, exports);
tslib_1.__exportStar(ModuleBuilder_1, exports);


});

unwrapExports(modules);

var BootModule_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });








/**
 * Bootstrap ext for ioc. auto run setup after registered.
 * with @IocExt('setup') decorator.
 * @export
 * @class BootModule
 */
let BootModule = class BootModule {
    constructor(container) {
        this.container = container;
    }
    /**
     * register aop for container.
     *
     * @memberof AopModule
     */
    setup() {
        let container = this.container;
        let lifeScope = container.get(core_1.LifeScopeToken);
        lifeScope.registerDecorator(Annotation.Annotation, core_1.CoreActions.bindProvider, core_1.CoreActions.cache, core_1.CoreActions.componentBeforeInit, core_1.CoreActions.componentInit, core_1.CoreActions.componentAfterInit);
        lifeScope.registerDecorator(DIModule.DIModule, core_1.CoreActions.bindProvider, core_1.CoreActions.cache, core_1.CoreActions.componentBeforeInit, core_1.CoreActions.componentInit, core_1.CoreActions.componentAfterInit);
        lifeScope.registerDecorator(Bootstrap.Bootstrap, core_1.CoreActions.bindProvider, core_1.CoreActions.cache, core_1.CoreActions.componentBeforeInit, core_1.CoreActions.componentInit, core_1.CoreActions.componentAfterInit);
        container.use(annotations, modules, boot);
    }
};
BootModule.classAnnations = { "name": "BootModule", "params": { "constructor": ["container"], "setup": [] } };
BootModule = tslib_1.__decorate([
    core_1.IocExt('setup'),
    tslib_1.__param(0, core_1.Inject(core_1.ContainerToken)),
    tslib_1.__metadata("design:paramtypes", [Object])
], BootModule);
exports.BootModule = BootModule;


});

unwrapExports(BootModule_1);
var BootModule_2 = BootModule_1.BootModule;

var RunnableBuilder_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });







/**
 * runnable events
 *
 * @export
 * @enum {number}
 */
var RunnableEvents;
(function (RunnableEvents) {
    /**
     * on root container created.
     */
    RunnableEvents["onRootContainerCreated"] = "onRootContainerCreated";
    /**
     * on root container inited.
     */
    RunnableEvents["onRootContainerInited"] = "onRootContainerInited";
    /**
     * on module created.
     */
    RunnableEvents["onModuleCreated"] = "onModuleCreated";
    /**
     * on registered runner use module.
     */
    RunnableEvents["registeredExt"] = "registeredExt";
})(RunnableEvents = exports.RunnableEvents || (exports.RunnableEvents = {}));
/**
 * runnable builder.
 *
 * @export
 * @class RunnableBuilder
 * @extends {ModuleBuilder}
 * @template T
 */
let RunnableBuilder = class RunnableBuilder extends modules.ModuleBuilder {
    constructor(baseURL) {
        super();
        this.inited = false;
        this._baseURL = baseURL;
        this.customRegs = [];
        this.globalModules = [];
        this.beforeInitPds = new Map();
        this.afterInitPds = new Map();
        this.events = new utils.Events();
        this.initEvents();
    }
    initEvents() {
        this.on(RunnableEvents.onRootContainerInited, (container) => {
            this.afterInitPds.forEach((val, key) => {
                container.bindProvider(key, val);
            });
        });
    }
    on(name, event) {
        this.events.on(name, event);
        return this;
    }
    off(name, event) {
        this.events.off(name, event);
        return this;
    }
    emit(name, ...args) {
        this.events.emit(name, ...args);
    }
    getPools() {
        if (!this.pools) {
            this.pools = new utils.ContainerPool(this.createContainerBuilder());
            this.createDefaultContainer();
        }
        return this.pools;
    }
    getRunRoot(container) {
        return this._baseURL || container.get(IRunnableBuilder.ProcessRunRootToken);
    }
    createContainerBuilder() {
        return new core_1.ContainerBuilder();
    }
    /**
     * use module as global Depdences module.
     *
     * @param {...LoadType[]} modules
     * @returns {this}
     * @memberof RunnableBuilder
     */
    use(...modules$$1) {
        this.globalModules = this.globalModules.concat(modules$$1);
        this.inited = false;
        return this;
    }
    /**
     * bind provider
     *
     * @template T
     * @param {Token<T>} provide
     * @param {Token<T> | Factory<T>} provider
     * @param {boolean} [beforeInit]
     * @returns {this}
     * @memberof RunnableBuilder
     */
    provider(provide, provider, beforeInit) {
        if (beforeInit) {
            this.beforeInitPds.set(provide, provider);
        }
        else {
            this.afterInitPds.set(provide, provider);
        }
        return this;
    }
    getProvider(provide, beforeInit) {
        if (beforeInit) {
            return this.beforeInitPds.has(provide) ? this.beforeInitPds.get(provide) : null;
        }
        else {
            return this.afterInitPds.has(provide) ? this.afterInitPds.get(provide) : null;
        }
    }
    async load(token, config, options) {
        await this.initRootContainer();
        return await super.load(token, config, options);
    }
    async bootstrap(token, config, options) {
        let params = this.vaildParams(token, config, options);
        options = params.options || {};
        let injmdl = params.token ? await this.load(params.token, params.config, options) : await this.load(params.config, options);
        options.env = injmdl;
        let builder = this.getBuilder(injmdl);
        return params.token ? await builder.bootstrap(params.token, params.config, options) : await builder.bootstrap(params.config, options);
    }
    /**
     * get module builder
     *
     * @param {(Token<T> | ModuleConfig<T>)} token
     * @param {ModuleEnv} [env]
     * @returns {IModuleBuilder<T>}
     * @memberof RunnableBuilder
     */
    async getBuilderByConfig(token, options) {
        let injmdl = await this.load(token, options);
        return this.getBuilder(injmdl);
    }
    getBuilder(injmdl) {
        let cfg = injmdl.config;
        let container = injmdl.container;
        let builder;
        if (cfg) {
            if (core_1.isClass(cfg.builder)) {
                if (!container.has(cfg.builder)) {
                    container.register(cfg.builder);
                }
            }
            if (core_1.isToken(cfg.builder)) {
                builder = container.resolve(cfg.builder);
            }
            else if (cfg.builder instanceof modules.ModuleBuilder) {
                builder = cfg.builder;
            }
        }
        let tko = injmdl.token;
        if (!builder && tko) {
            builder = container.getService(modules.ModuleBuilderToken, tko, (tk) => new modules.InjectModuleBuilderToken(tk), cfg.defaultBuilder || modules.ModuleBuilderToken);
        }
        return builder || this;
    }
    getConfigManager() {
        if (!this.configMgr) {
            this.configMgr = this.createConfigureMgr();
        }
        return this.configMgr;
    }
    createConfigureMgr() {
        let container = this.getPools().getDefault();
        return container.getService(IConfigureManager.ConfigureMgrToken, core_1.lang.getClass(this), { baseURL: this.getRunRoot(container) });
    }
    createDefaultContainer() {
        let container = this.pools.getDefault();
        container.register(BootModule_1.BootModule);
        let chain = container.getBuilder().getInjectorChain(container);
        chain.first(container.resolve(modules.DIModuleInjectorToken));
        container.bindProvider(utils.ContainerPoolToken, () => this.getPools());
        this.beforeInitPds.forEach((val, key) => {
            container.bindProvider(key, val);
        });
        this.events.emit(RunnableEvents.onRootContainerCreated, container);
        return container;
    }
    async initRootContainer() {
        if (this.inited) {
            return;
        }
        let container = this.getPools().getDefault();
        await this.registerExts(container);
        let configManager = this.getConfigManager();
        let config = await configManager.getConfig();
        await this.registerByConfigure(container, config);
        await configManager.bindBuilder(this);
        this.inited = true;
        this.events.emit(RunnableEvents.onRootContainerInited, container);
    }
    /**
     * register ioc exts
     *
     * @protected
     * @param {IContainer} container
     * @param {AppConfigure} config
     * @memberof RunnableBuilder
     */
    async registerExts(container) {
        if (this.globalModules.length) {
            let usedModules = this.globalModules;
            let types = await container.loadModule(...usedModules);
            this.emit(RunnableEvents.registeredExt, types, container);
        }
    }
    /**
     * register by configure.
     *
     * @protected
     * @param {IContainer} container
     * @param {ModuleConfig<T>} config
     * @returns {Promise<void>}
     * @memberof RunnableBuilder
     */
    async registerByConfigure(container, config) {
        config.baseURL = this.getRunRoot(container);
        await core_1.PromiseUtil.step(this.customRegs.map(async (cs) => {
            let tokens = await cs(container, config, this);
            return tokens;
        }));
    }
};
RunnableBuilder.classAnnations = { "name": "RunnableBuilder", "params": { "constructor": ["baseURL"], "initEvents": [], "on": ["name", "event"], "off": ["name", "event"], "emit": ["name", "args"], "getPools": [], "getRunRoot": ["container"], "createContainerBuilder": [], "use": ["modules"], "provider": ["provide", "provider", "beforeInit"], "getProvider": ["provide", "beforeInit"], "load": ["token", "config", "options"], "bootstrap": ["token", "config", "options"], "getBuilderByConfig": ["token", "options"], "getBuilder": ["injmdl"], "getConfigManager": [], "createConfigureMgr": [], "createDefaultContainer": [], "initRootContainer": [], "registerExts": ["container"], "registerByConfigure": ["container", "config"] } };
RunnableBuilder = tslib_1.__decorate([
    core_1.Injectable(IRunnableBuilder.RunnableBuilderToken),
    tslib_1.__metadata("design:paramtypes", [String])
], RunnableBuilder);
exports.RunnableBuilder = RunnableBuilder;


});

unwrapExports(RunnableBuilder_1);
var RunnableBuilder_2 = RunnableBuilder_1.RunnableEvents;
var RunnableBuilder_3 = RunnableBuilder_1.RunnableBuilder;

var IApplicationBuilder = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 *  application builder token.
 */
exports.ApplicationBuilderToken = new core_1.InjectToken('DI_AppBuilder');


});

unwrapExports(IApplicationBuilder);
var IApplicationBuilder_1 = IApplicationBuilder.ApplicationBuilderToken;

var AppConfigure = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 * application configuration token.
 */
exports.AppConfigureToken = new core_1.InjectToken('DI_APP_Configuration');


});

unwrapExports(AppConfigure);
var AppConfigure_1 = AppConfigure.AppConfigureToken;

var ApplicationBuilder_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });




/**
 * application events
 *
 * @export
 * @enum {number}
 */
exports.AppEvents = RunnableBuilder_1.RunnableEvents;
/**
 * application events
 */
exports.ApplicationEvents = RunnableBuilder_1.RunnableEvents;
/**
 * application builder.
 *
 * @export
 * @class Default ApplicationBuilder
 * @extends {ModuleBuilder}
 * @template T
 */
let ApplicationBuilder = class ApplicationBuilder extends RunnableBuilder_1.RunnableBuilder {
    constructor(baseURL) {
        super(baseURL);
        this.configs = [];
    }
    /**
     * use configuration.
     *
     * @param {(string | AppConfigure)} [config]
     * @returns {this} global config for this application.
     * @memberof Bootstrap
     */
    useConfiguration(config) {
        if (this.configMgr) {
            this.configMgr.useConfiguration(config);
        }
        this.configs.push(config);
        return this;
    }
    createConfigureMgr() {
        let cfgMgr = super.createConfigureMgr();
        if (this.configs && this.configs.length) {
            this.configs.forEach(cfg => cfgMgr.useConfiguration(cfg));
        }
        return cfgMgr;
    }
    /**
     * register by configure.
     *
     * @protected
     * @param {IContainer} container
     * @param {AppConfigure} config
     * @returns {Promise<void>}
     * @memberof RunnableBuilder
     */
    async registerByConfigure(container, config) {
        await super.registerByConfigure(container, config);
    }
};
ApplicationBuilder.classAnnations = { "name": "ApplicationBuilder", "params": { "constructor": ["baseURL"], "useConfiguration": ["config"], "createConfigureMgr": [], "registerByConfigure": ["container", "config"] } };
ApplicationBuilder = tslib_1.__decorate([
    core_1.Injectable(IApplicationBuilder.ApplicationBuilderToken),
    tslib_1.__metadata("design:paramtypes", [String])
], ApplicationBuilder);
exports.ApplicationBuilder = ApplicationBuilder;


});

unwrapExports(ApplicationBuilder_1);
var ApplicationBuilder_2 = ApplicationBuilder_1.AppEvents;
var ApplicationBuilder_3 = ApplicationBuilder_1.ApplicationEvents;
var ApplicationBuilder_4 = ApplicationBuilder_1.ApplicationBuilder;

var boot = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(IConfigureManager, exports);
tslib_1.__exportStar(ConfigureManager_1, exports);
tslib_1.__exportStar(IRunnableBuilder, exports);
tslib_1.__exportStar(RunnableBuilder_1, exports);
tslib_1.__exportStar(IApplicationBuilder, exports);
tslib_1.__exportStar(AppConfigure, exports);
tslib_1.__exportStar(ApplicationBuilder_1, exports);


});

unwrapExports(boot);

var Bootstrap = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });



/**
 * create bootstrap decorator.
 *
 * @export
 * @template T
 * @param {string} name
 * @param {Token<IRunnableBuilder<any>>>} [builder] default builder
 * @param {Token<IAnnotationBuilder<any>>} [defaultAnnoBuilder] default type builder.
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IBootstrapDecorator<T>}
 */
function createBootstrapDecorator(name, defaultBuilder, defaultAnnoBuilder, adapter, metadataExtends) {
    return DIModule.createDIModuleDecorator(name, defaultBuilder, defaultAnnoBuilder, adapter, (metadata) => {
        if (metadataExtends) {
            metadataExtends(metadata);
        }
        if (metadata.builder) {
            setTimeout(() => {
                new boot.ApplicationBuilder()
                    .use(...(metadata.globals || []))
                    .bootstrap(metadata);
            }, 300);
        }
        return metadata;
    });
}
exports.createBootstrapDecorator = createBootstrapDecorator;
/**
 * Bootstrap Decorator, definde class as mvc bootstrap module.
 *
 * @Bootstrap
 */
exports.Bootstrap = createBootstrapDecorator('Bootstrap', boot.ApplicationBuilderToken, IAnnotationBuilder.AnnotationBuilderToken);


});

unwrapExports(Bootstrap);
var Bootstrap_1 = Bootstrap.createBootstrapDecorator;
var Bootstrap_2 = Bootstrap.Bootstrap;

var decorators = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(Annotation, exports);
tslib_1.__exportStar(DIModule, exports);
tslib_1.__exportStar(Bootstrap, exports);


});

unwrapExports(decorators);

var D__Workspace_github_node_tsioc_packages_bootstrap_esnext = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(decorators, exports);
tslib_1.__exportStar(boot, exports);
tslib_1.__exportStar(annotations, exports);
tslib_1.__exportStar(modules, exports);
tslib_1.__exportStar(runnable, exports);
tslib_1.__exportStar(utils, exports);
tslib_1.__exportStar(BootModule_1, exports);


});

var index$6 = unwrapExports(D__Workspace_github_node_tsioc_packages_bootstrap_esnext);

return index$6;

})));

//# sourceMappingURL=sourcemaps/bootstrap.js.map
