"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var core_1 = require("@ts-ioc/core");
var IConfigureManager_1 = require("./IConfigureManager");
/**
 * configure manager.
 *
 * @export
 * @class ConfigureManager
 */
// @RefTo(RunnableBuilderToken)
var ConfigureManager = /** @class */ (function () {
    /**
     * Creates an instance of ConfigureManager.
     * @param {string} [baseURL]
     * @memberof ConfigureManager
     */
    function ConfigureManager(baseURL) {
        this.baseURL = baseURL;
        this.configs = [];
    }
    /**
     * use configuration.
     *
     * @param {(string | AppConfigure)} [config]
     * @returns {this} this configure manager.
     * @memberof Bootstrap
     */
    ConfigureManager.prototype.useConfiguration = function (config) {
        if (core_1.isUndefined(config)) {
            config = '';
        }
        // clean cached config.
        this.config = null;
        var idx = this.configs.indexOf(config);
        if (idx >= 0) {
            this.configs.splice(idx, 1);
        }
        this.configs.push(config);
        return this;
    };
    /**
     * bind runnable builder.
     *
     * @param {IRunnableBuilder<any>} builder
     * @memberof ConfigureManager
     */
    ConfigureManager.prototype.bindBuilder = function (builder) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/];
            });
        });
    };
    /**
     * get config.
     *
     * @returns {Promise<T>}
     * @memberof ConfigureManager
     */
    ConfigureManager.prototype.getConfig = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (!!this.config) return [3 /*break*/, 2];
                        _a = this;
                        return [4 /*yield*/, this.initConfig()];
                    case 1:
                        _a.config = _b.sent();
                        _b.label = 2;
                    case 2: return [2 /*return*/, this.config];
                }
            });
        });
    };
    /**
     * init config.
     *
     * @protected
     * @returns
     * @memberof ConfigureManager
     */
    ConfigureManager.prototype.initConfig = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var config, exts;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getDefaultConfig()];
                    case 1:
                        config = _a.sent();
                        if (this.configs.length < 1) {
                            this.configs.push(''); // load default loader config.
                        }
                        return [4 /*yield*/, Promise.all(this.configs.map(function (cfg) {
                                if (core_1.isString(cfg)) {
                                    return _this.loadConfig(cfg);
                                }
                                else {
                                    return cfg;
                                }
                            }))];
                    case 2:
                        exts = _a.sent();
                        exts.forEach(function (exCfg) {
                            if (exCfg) {
                                core_1.lang.assign(config, exCfg);
                            }
                        });
                        return [2 /*return*/, config];
                }
            });
        });
    };
    /**
     * load config.
     *
     * @protected
     * @param {string} src
     * @returns {Promise<T>}
     * @memberof ConfigureManager
     */
    ConfigureManager.prototype.loadConfig = function (src) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var loader, builder, cfg;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.container.has(IConfigureManager_1.ConfigureLoaderToken)) return [3 /*break*/, 2];
                        loader = this.container.resolve(IConfigureManager_1.ConfigureLoaderToken, { baseURL: this.baseURL, container: this.container });
                        return [4 /*yield*/, loader.load(src)];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2:
                        if (!src) return [3 /*break*/, 4];
                        builder = this.container.getBuilder();
                        return [4 /*yield*/, builder.loader.load([src])];
                    case 3:
                        cfg = _a.sent();
                        return [2 /*return*/, cfg.length ? cfg[0] : null];
                    case 4: return [2 /*return*/, null];
                }
            });
        });
    };
    /**
     * get default config.
     *
     * @protected
     * @returns {Promise<T>}
     * @memberof ConfigureManager
     */
    ConfigureManager.prototype.getDefaultConfig = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                if (this.container.has(IConfigureManager_1.DefaultConfigureToken)) {
                    return [2 /*return*/, this.container.resolve(IConfigureManager_1.DefaultConfigureToken)];
                }
                else {
                    return [2 /*return*/, {}];
                }
                return [2 /*return*/];
            });
        });
    };
    ConfigureManager.classAnnations = { "name": "ConfigureManager", "params": { "constructor": ["baseURL"], "useConfiguration": ["config"], "bindBuilder": ["builder"], "getConfig": [], "initConfig": [], "loadConfig": ["src"], "getDefaultConfig": [] } };
    tslib_1.__decorate([
        core_1.Inject(core_1.ContainerToken),
        tslib_1.__metadata("design:type", Object)
    ], ConfigureManager.prototype, "container", void 0);
    ConfigureManager = tslib_1.__decorate([
        core_1.Injectable(IConfigureManager_1.ConfigureMgrToken),
        tslib_1.__metadata("design:paramtypes", [String])
    ], ConfigureManager);
    return ConfigureManager;
}());
exports.ConfigureManager = ConfigureManager;

//# sourceMappingURL=../sourcemaps/boot/ConfigureManager.js.map
