import { IContainer, LoadType, Factory, Token, IContainerBuilder } from '@ts-ioc/core';
import { IRunnableBuilder, CustomRegister } from './IRunnableBuilder';
import { ModuleBuilder, InjectedModule, IModuleBuilder, ModuleConfig, ModuleConfigure, BootOptions } from '../modules';
import { ContainerPool, Events, IEvents } from '../utils';
import { Runnable } from '../runnable';
import { IConfigureManager } from './IConfigureManager';
/**
 * runnable events
 *
 * @export
 * @enum {number}
 */
export declare enum RunnableEvents {
    /**
     * on root container created.
     */
    onRootContainerCreated = "onRootContainerCreated",
    /**
     * on root container inited.
     */
    onRootContainerInited = "onRootContainerInited",
    /**
     * on module created.
     */
    onModuleCreated = "onModuleCreated",
    /**
     * on registered runner use module.
     */
    registeredExt = "registeredExt"
}
/**
 * runnable builder.
 *
 * @export
 * @class RunnableBuilder
 * @extends {ModuleBuilder}
 * @template T
 */
export declare class RunnableBuilder<T> extends ModuleBuilder<T> implements IRunnableBuilder<T>, IEvents {
    protected globalModules: LoadType[];
    protected customRegs: CustomRegister<T>[];
    protected beforeInitPds: Map<Token<any>, any>;
    protected afterInitPds: Map<Token<any>, any>;
    protected configMgr: IConfigureManager<ModuleConfig<T>>;
    inited: boolean;
    events: Events;
    private _baseURL;
    constructor(baseURL?: string);
    protected initEvents(): void;
    on(name: string, event: (...args: any[]) => void): this;
    off(name: string, event?: (...args: any[]) => void): this;
    emit(name: string, ...args: any[]): void;
    getPools(): ContainerPool;
    getRunRoot(container: IContainer): string;
    protected createContainerBuilder(): IContainerBuilder;
    /**
     * use module as global Depdences module.
     *
     * @param {...LoadType[]} modules
     * @returns {this}
     * @memberof RunnableBuilder
     */
    use(...modules: LoadType[]): this;
    /**
     * bind provider
     *
     * @template T
     * @param {Token<T>} provide
     * @param {Token<T> | Factory<T>} provider
     * @param {boolean} [beforeInit]
     * @returns {this}
     * @memberof RunnableBuilder
     */
    provider(provide: Token<any>, provider: Token<any> | Factory<any>, beforeInit?: boolean): this;
    getProvider(provide: Token<any>, beforeInit?: boolean): Token<any> | Factory<any>;
    load(token: Token<T> | ModuleConfigure, config?: ModuleConfig<T> | BootOptions<T>, options?: BootOptions<T>): Promise<InjectedModule<T>>;
    bootstrap(token: Token<T> | ModuleConfigure, config?: ModuleConfig<T> | BootOptions<T>, options?: BootOptions<T>): Promise<Runnable<T>>;
    /**
     * get module builder
     *
     * @param {(Token<T> | ModuleConfig<T>)} token
     * @param {ModuleEnv} [env]
     * @returns {IModuleBuilder<T>}
     * @memberof RunnableBuilder
     */
    getBuilderByConfig(token: Token<T> | ModuleConfig<T>, options?: BootOptions<T>): Promise<IModuleBuilder<T>>;
    getBuilder(injmdl: InjectedModule<T>): IModuleBuilder<T>;
    getConfigManager(): IConfigureManager<ModuleConfig<T>>;
    protected createConfigureMgr(): IConfigureManager<ModuleConfigure>;
    protected createDefaultContainer(): IContainer;
    protected initRootContainer(): Promise<void>;
    /**
     * register ioc exts
     *
     * @protected
     * @param {IContainer} container
     * @param {AppConfigure} config
     * @memberof RunnableBuilder
     */
    protected registerExts(container: IContainer): Promise<void>;
    /**
     * register by configure.
     *
     * @protected
     * @param {IContainer} container
     * @param {ModuleConfig<T>} config
     * @returns {Promise<void>}
     * @memberof RunnableBuilder
     */
    protected registerByConfigure(container: IContainer, config: ModuleConfig<T>): Promise<void>;
    static classAnnations: any;
}
