"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@ts-ioc/core");
var IAnnotationBuilder_1 = require("../annotations/IAnnotationBuilder");
/**
 * create type builder decorator
 *
 * @export
 * @template T
 * @param {string} name
 * @param {string} [decorType]
 * @param {(Token<IAnnotationBuilder<any>> | IAnnotationBuilder<any>)} [defaultBuilder]
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IAnnotationDecorator<T>}
 */
function createAnnotationDecorator(name, defaultBuilder, adapter, metadataExtends) {
    return core_1.createClassDecorator(name, function (args) {
        if (adapter) {
            adapter(args);
        }
    }, function (metadata) {
        if (metadataExtends) {
            metadata = metadataExtends(metadata);
        }
        if (defaultBuilder && !metadata.defaultAnnoBuilder) {
            metadata.defaultAnnoBuilder = defaultBuilder;
        }
        return metadata;
    });
}
exports.createAnnotationDecorator = createAnnotationDecorator;
/**
 * Annotation decorator, use to define class build way via config.
 *
 * @Annotation
 */
exports.Annotation = createAnnotationDecorator('Annotation', IAnnotationBuilder_1.AnnotationBuilderToken);

//# sourceMappingURL=../sourcemaps/decorators/Annotation.js.map
