"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var IAnnotationBuilder_1 = require("../annotations/IAnnotationBuilder");
var DIModule_1 = require("./DIModule");
var boot_1 = require("../boot");
/**
 * create bootstrap decorator.
 *
 * @export
 * @template T
 * @param {string} name
 * @param {Token<IRunnableBuilder<any>>>} [builder] default builder
 * @param {Token<IAnnotationBuilder<any>>} [defaultAnnoBuilder] default type builder.
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IBootstrapDecorator<T>}
 */
function createBootstrapDecorator(name, defaultBuilder, defaultAnnoBuilder, adapter, metadataExtends) {
    return DIModule_1.createDIModuleDecorator(name, defaultBuilder, defaultAnnoBuilder, adapter, function (metadata) {
        if (metadataExtends) {
            metadataExtends(metadata);
        }
        if (metadata.builder) {
            setTimeout(function () {
                var _a;
                (_a = new boot_1.ApplicationBuilder()).use.apply(_a, (metadata.globals || [])).bootstrap(metadata);
            }, 300);
        }
        return metadata;
    });
}
exports.createBootstrapDecorator = createBootstrapDecorator;
/**
 * Bootstrap Decorator, definde class as mvc bootstrap module.
 *
 * @Bootstrap
 */
exports.Bootstrap = createBootstrapDecorator('Bootstrap', boot_1.ApplicationBuilderToken, IAnnotationBuilder_1.AnnotationBuilderToken);

//# sourceMappingURL=../sourcemaps/decorators/Bootstrap.js.map
