import { Type, IContainer, ModuleInjector, InjectModuleInjectorToken, IModuleValidate, Token, ParamProviders, IModuleInjector, IMetaAccessor } from '@ts-ioc/core';
import { ModuleConfigure } from './ModuleConfigure';
import { InjectedModule } from './InjectedModule';
/**
 * DIModule injector interface.
 *
 * @export
 * @interface IDIModuleInjector
 * @extends {IModuleInjector}
 */
export interface IDIModuleInjector extends IModuleInjector {
    /**
     * get meta accessor.
     *
     * @template T
     * @param {IContainer} container
     * @param {Token<T>} token
     * @returns {IMetaAccessor<T>}
     * @memberof IDIModuleInjector
     */
    getMetaAccessor<T>(container: IContainer, token: Token<T> | Token<T>[]): IMetaAccessor<T>;
}
/**
 * DIModule injector token.
 */
export declare const DIModuleInjectorToken: InjectModuleInjectorToken<IDIModuleInjector>;
/**
 * DIModule injector.
 *
 * @export
 * @class DIModuleInjector
 * @extends {ModuleInjector}
 */
export declare class DIModuleInjector extends ModuleInjector implements IDIModuleInjector {
    constructor(validate: IModuleValidate);
    protected setup(container: IContainer, type: Type<any>): Promise<void>;
    protected syncSetup(container: IContainer, type: Type<any>): void;
    protected valid(container: IContainer, type: Type<any>): boolean;
    getMetaAccessor<T>(container: IContainer, token: Token<T> | Token<T>[]): IMetaAccessor<T>;
    protected importModule(container: IContainer, type: Type<any>): Promise<InjectedModule<any>>;
    protected registerConfgureDepds(container: IContainer, config: ModuleConfigure, type?: Type<any>): Promise<ModuleConfigure>;
    protected importConfigExports(container: IContainer, providerContainer: IContainer, injMd: InjectedModule<any>): Promise<IContainer>;
    protected bindProvider(container: IContainer, providers: ParamProviders[]): Token<any>[];
    static classAnnations: any;
}
