"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var core_1 = require("@ts-ioc/core");
var DIModuleValidate_1 = require("./DIModuleValidate");
var DIModule_1 = require("../decorators/DIModule");
var utils_1 = require("../utils");
var InjectedModule_1 = require("./InjectedModule");
/**
 * DIModule injector token.
 */
exports.DIModuleInjectorToken = new core_1.InjectModuleInjectorToken(DIModule_1.DIModule.toString());
/**
 * DIModule injector.
 *
 * @export
 * @class DIModuleInjector
 * @extends {ModuleInjector}
 */
var DIModuleInjector = /** @class */ (function (_super) {
    tslib_1.__extends(DIModuleInjector, _super);
    function DIModuleInjector(validate) {
        return _super.call(this, validate) || this;
    }
    DIModuleInjector.prototype.setup = function (container, type) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.importModule(container, type)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    DIModuleInjector.prototype.syncSetup = function (container, type) {
        this.importModule(container, type);
    };
    DIModuleInjector.prototype.valid = function (container, type) {
        if (!this.validate) {
            return true;
        }
        return this.validate.valid(type);
    };
    DIModuleInjector.prototype.getMetaAccessor = function (container, token) {
        return container.getService(core_1.MetaAccessorToken, token);
    };
    DIModuleInjector.prototype.importModule = function (container, type) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var pools, newContainer, builder, decorator, accor, metaConfig, exps, classProvides, injMd;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        pools = container.get(utils_1.ContainerPoolToken);
                        newContainer = pools.create(container);
                        newContainer.register(type);
                        builder = newContainer.getBuilder();
                        decorator = this.validate.getDecorator();
                        accor = this.getMetaAccessor(newContainer, decorator);
                        metaConfig = accor.getMetadata(type, newContainer, undefined, decorator ? function (dec) { return dec === decorator; } : undefined);
                        return [4 /*yield*/, this.registerConfgureDepds(newContainer, metaConfig, type)];
                    case 1:
                        metaConfig = _a.sent();
                        exps = [].concat.apply([], builder.loader.getTypes(metaConfig.exports || []));
                        classProvides = [];
                        exps.forEach(function (ty) {
                            var classPd = newContainer.resolveValue(new core_1.InjectClassProvidesToken(ty));
                            if (classPd && core_1.isArray(classPd.provides) && classPd.provides.length) {
                                classProvides = classProvides.concat(classPd.provides);
                            }
                        });
                        injMd = new InjectedModule_1.InjectedModule(metaConfig.token || type, metaConfig, newContainer, type, exps);
                        container.bindProvider(new InjectedModule_1.InjectedModuleToken(type), injMd);
                        return [4 /*yield*/, this.importConfigExports(container, newContainer, injMd)];
                    case 2:
                        _a.sent();
                        return [2 /*return*/, injMd];
                }
            });
        });
    };
    DIModuleInjector.prototype.registerConfgureDepds = function (container, config, type) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(core_1.isArray(config.imports) && config.imports.length)) return [3 /*break*/, 2];
                        return [4 /*yield*/, container.loadModule.apply(container, config.imports)];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2:
                        if (!(!type && core_1.isArray(config.providers) && config.providers.length)) return [3 /*break*/, 4];
                        return [4 /*yield*/, this.bindProvider(container, config.providers)];
                    case 3:
                        _a.sent();
                        _a.label = 4;
                    case 4: return [2 /*return*/, config];
                }
            });
        });
    };
    DIModuleInjector.prototype.importConfigExports = function (container, providerContainer, injMd) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var chain_1;
            return tslib_1.__generator(this, function (_a) {
                if (container === providerContainer) {
                    return [2 /*return*/, container];
                }
                if (injMd) {
                    chain_1 = container.getResolvers();
                    chain_1.next(injMd);
                    if (injMd.exports && injMd.exports.length) {
                        providerContainer.getResolvers().toArray().filter(function (r) {
                            if (r instanceof core_1.Container) {
                                return false;
                            }
                            else {
                                return injMd.exports.indexOf(r.type) >= 0;
                            }
                        }).forEach(function (r) {
                            chain_1.next(r);
                        });
                    }
                }
                return [2 /*return*/, container];
            });
        });
    };
    DIModuleInjector.prototype.bindProvider = function (container, providers) {
        var parser = container.getProviderParser();
        var pdrmap = parser.parse.apply(parser, providers);
        var tokens = pdrmap.provides();
        tokens.forEach(function (key) {
            container.bindProvider(key, function () {
                var providers = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    providers[_i] = arguments[_i];
                }
                return pdrmap.resolve.apply(pdrmap, [key].concat(providers));
            });
        });
        return tokens;
    };
    DIModuleInjector.classAnnations = { "name": "DIModuleInjector", "params": { "constructor": ["validate"], "setup": ["container", "type"], "syncSetup": ["container", "type"], "valid": ["container", "type"], "getMetaAccessor": ["container", "token"], "importModule": ["container", "type"], "registerConfgureDepds": ["container", "config", "type"], "importConfigExports": ["container", "providerContainer", "injMd"], "bindProvider": ["container", "providers"] } };
    DIModuleInjector = tslib_1.__decorate([
        core_1.Singleton(exports.DIModuleInjectorToken),
        tslib_1.__param(0, core_1.Inject(DIModuleValidate_1.DIModuleValidateToken)),
        tslib_1.__metadata("design:paramtypes", [Object])
    ], DIModuleInjector);
    return DIModuleInjector;
}(core_1.ModuleInjector));
exports.DIModuleInjector = DIModuleInjector;

//# sourceMappingURL=../sourcemaps/modules/DIModuleInjector.js.map
