"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var core_1 = require("@ts-ioc/core");
/**
 * injected module.
 *
 * @export
 * @class InjectedModule
 * @template T
 */
var InjectedModule = /** @class */ (function () {
    function InjectedModule(token, config, container, type, exports) {
        this.token = token;
        this.config = config;
        this.container = container;
        this.type = type;
        this.exports = exports;
    }
    InjectedModule.prototype.getProviderMap = function () {
        if (!this._map) {
            this._map = this.container.resolveValue(new core_1.InjectReference(core_1.ProviderMap, this.type || this.token));
        }
        return this._map;
    };
    InjectedModule.prototype.resolve = function (token) {
        var providers = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            providers[_i - 1] = arguments[_i];
        }
        var _a;
        var pdr = this.getProviderMap();
        if (pdr && pdr.hasRegister(token)) {
            return pdr.resolve.apply(pdr, [token].concat(providers));
        }
        else {
            return (_a = this.container).resolveValue.apply(_a, [token].concat(providers));
        }
    };
    InjectedModule.prototype.hasRegister = function (key) {
        if (this.container.hasRegister(key)) {
            return true;
        }
        else {
            var pdr = this.getProviderMap();
            return pdr && pdr.hasRegister(key);
        }
    };
    InjectedModule.classAnnations = { "name": "InjectedModule", "params": { "constructor": ["token", "config", "container", "type", "exports"], "getProviderMap": [], "resolve": ["token", "providers"], "hasRegister": ["key"] } };
    return InjectedModule;
}());
exports.InjectedModule = InjectedModule;
/**
 * Injected Module Token.
 *
 * @export
 * @class InjectModuleMetaConfigToken
 * @extends {Registration<Type<T>>}
 * @template T
 */
var InjectedModuleToken = /** @class */ (function (_super) {
    tslib_1.__extends(InjectedModuleToken, _super);
    function InjectedModuleToken(type) {
        return _super.call(this, type, 'InjectedModule') || this;
    }
    InjectedModuleToken.classAnnations = { "name": "InjectedModuleToken", "params": { "constructor": ["type"] } };
    return InjectedModuleToken;
}(core_1.Registration));
exports.InjectedModuleToken = InjectedModuleToken;

//# sourceMappingURL=../sourcemaps/modules/InjectedModule.js.map
