import { Token } from '@ts-ioc/core';
import { ModuleConfigure } from '../modules';
/**
 * application runer.
 *
 * @export
 * @interface IRunner
 * @template T
 */
export interface IRunner<T> {
    /**
     * target instance.
     *
     * @type {T}
     * @memberof IRunner
     */
    getTarget?(): T;
    /**
     * get target token.
     *
     * @returns {Token<T>}
     * @memberof IRunner
     */
    getTargetToken?(): Token<T>;
    /**
     * run application via boot instance.
     *
     * @param {*} [data]
     * @returns {Promise<any>}
     * @memberof IRunner
     */
    run(data?: any): Promise<any>;
}
/**
 * boot element.
 *
 * @export
 * @abstract
 * @class Boot
 * @implements {IBoot}
 */
export declare abstract class Runner<T> implements IRunner<T> {
    protected token?: Token<T>;
    protected instance?: T;
    protected config?: ModuleConfigure;
    constructor(token?: Token<T>, instance?: T, config?: ModuleConfigure);
    getTarget?(): T;
    getTargetToken?(): Token<T>;
    /**
     * run boot.
     *
     * @abstract
     * @param {*} [data]
     * @returns {Promise<any>}
     * @memberof Runner
     */
    abstract run(data?: any): Promise<any>;
    static classAnnations: any;
}
/**
 * boot element
 *
 * @export
 * @class Boot
 * @extends {Runner<any>}
 */
export declare abstract class Boot extends Runner<any> {
    protected token?: Token<any>;
    protected instance?: any;
    protected config?: ModuleConfigure;
    constructor(token?: Token<any>, instance?: any, config?: ModuleConfigure);
    static classAnnations: any;
}
/**
 * target is runner or not.
 *
 * @export
 * @param {*} target
 * @returns {target is IRunner<any>}
 */
export declare function isRunner(target: any): target is IRunner<any>;
